package ch.unige.aou.controller.admin;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.ResourceController;
import ch.unige.solidify.exception.SolidifyHttpErrorException;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.security.AdminPermissions;
import ch.unige.solidify.security.EveryonePermissions;

import ch.unige.aou.business.GlobalBannerService;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.settings.GlobalBanner;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;

@AdminPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_GLOBAL_BANNERS)
public class GlobalBannerController extends ResourceController<GlobalBanner> {

  @PostMapping
  public HttpEntity<GlobalBanner> create(@RequestBody GlobalBanner globalBanner) {
    return super.create(globalBanner);
  }

  @PatchMapping(SolidifyConstants.URL_ID)
  public HttpEntity<GlobalBanner> update(@PathVariable String id, @RequestBody GlobalBanner globalBanner) {
    return super.update(id, globalBanner);
  }

  @Override
  public HttpEntity<Collection<GlobalBanner>> list(GlobalBanner search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpEntity<GlobalBanner> get(@PathVariable String id) {
    return super.get(id);
  }

  @EveryonePermissions
  @GetMapping(AouActionName.GET_ACTIVE)
  public HttpEntity<GlobalBanner> getActive() {
    GlobalBanner globalBanner = ((GlobalBannerService) this.itemService).findActive();
    if (globalBanner != null) {
      return new ResponseEntity<>(globalBanner, HttpStatus.OK);
    }
    return new ResponseEntity<>(HttpStatus.NOT_FOUND);
  }

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }
}
