package ch.unige.aou.controller.admin;

import java.io.IOException;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.security.AdminPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.controller.AouResourceWithLogoController;
import ch.unige.aou.model.settings.Institution;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;

@AdminPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_INSTITUTIONS)
public class InstitutionController extends AouResourceWithLogoController<Institution> {

  @Override
  public HttpEntity<Institution> create(@RequestBody Institution institution) {
    return super.create(institution);
  }

  @Override
  public HttpEntity<Collection<Institution>> list(@ModelAttribute Institution search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @Override
  public HttpStatus deleteList(@RequestBody String[] ids) {
    return super.deleteList(ids);
  }

  @Override
  public HttpEntity<Institution> update(@PathVariable String id, @RequestBody Institution institution) {
    return super.update(id, institution);
  }

  @PostMapping(SolidifyConstants.URL_ID_PLUS_SEP + AouActionName.UPLOAD_LOGO)
  @PreAuthorize("@organizationalUnitPermissionService.isAllowed(#id, 'UPDATE')")
  @Override
  public HttpEntity<Institution> uploadLogo(@PathVariable String id, @RequestParam("file") MultipartFile file) throws IOException {
    return super.uploadLogo(id, file);
  }

  @GetMapping(SolidifyConstants.URL_ID_PLUS_SEP + AouActionName.DOWNLOAD_LOGO)
  @ResponseBody
  @Override
  public HttpEntity<StreamingResponseBody> downloadLogo(@PathVariable String id) throws IOException {
    return super.downloadLogo(id);
  }

  @DeleteMapping(SolidifyConstants.URL_ID_PLUS_SEP + AouActionName.DELETE_LOGO)
  @ResponseBody
  @Override
  public HttpEntity<Institution> deleteLogo(@PathVariable String id) throws IOException {
    return super.deleteLogo(id);
  }

}
