package ch.unige.aou.controller.admin;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.ResourceController;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.security.AdminPermissions;
import ch.unige.solidify.security.UserPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.publication.PublicationSubtype;
import ch.unige.aou.model.publication.PublicationSubtypeContributorRole;
import ch.unige.aou.model.publication.PublicationSubtypeContributorRoleDTO;
import ch.unige.aou.model.publication.PublicationSubtypeDocumentFileType;
import ch.unige.aou.model.publication.PublicationSubtypeDocumentFileTypeDTO;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;

@AdminPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@SuppressWarnings("squid:S4684")
@RequestMapping(UrlPath.ADMIN_PUBLICATION_SUBTYPES)
public class PublicationSubtypeController extends ResourceController<PublicationSubtype> {

  @Override
  @PostMapping
  public HttpEntity<PublicationSubtype> create(@RequestBody PublicationSubtype subtype) {
    return super.create(subtype);
  }

  @Override
  @DeleteMapping(SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @Override
  @DeleteMapping
  public HttpStatus deleteList(@RequestBody String[] ids) {
    return super.deleteList(ids);
  }

  @Override
  @PatchMapping(SolidifyConstants.URL_ID)
  public HttpEntity<PublicationSubtype> update(@PathVariable String id, @RequestBody PublicationSubtype subtype) {
    return super.update(id, subtype);
  }

  @Override
  @UserPermissions
  public HttpEntity<Collection<PublicationSubtype>> list(@ModelAttribute PublicationSubtype search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  @UserPermissions
  public HttpEntity<PublicationSubtype> get(@PathVariable String id) {
    return super.get(id);
  }

  @UserPermissions
  @GetMapping(SolidifyConstants.URL_ID + SolidifyConstants.URL_SEP + AouActionName.LIST_DOCUMENT_FILE_TYPES)
  public HttpEntity<Collection<PublicationSubtypeDocumentFileTypeDTO>> listDocumentFileTypes(@PathVariable String id,
          @ModelAttribute PublicationSubtypeDocumentFileType filterItem) {

    PublicationSubtype publicationSubtype = this.itemService.findOne(id);
    List<PublicationSubtypeDocumentFileType> items = publicationSubtype.getPublicationSubtypeDocumentFileTypes();
    List<PublicationSubtypeDocumentFileTypeDTO> dtoItems = items.stream()
            .map(PublicationSubtypeDocumentFileTypeDTO::new)
            .sorted(Comparator.comparingInt(PublicationSubtypeDocumentFileTypeDTO::getSortValue))
            .collect(Collectors.toList());

    if (filterItem.getLevel() != null) {
      dtoItems = dtoItems.stream()
              .filter(item -> item.getLevel() == filterItem.getLevel())
              .collect(Collectors.toList());
    }

    return new ResponseEntity<>(new Collection(dtoItems), HttpStatus.OK);
  }

  @UserPermissions
  @GetMapping(SolidifyConstants.URL_ID + SolidifyConstants.URL_SEP + AouActionName.LIST_CONTRIBUTOR_ROLES)
  public HttpEntity<Collection<PublicationSubtypeContributorRoleDTO>> listContributorRoles(@PathVariable String id,
          @ModelAttribute PublicationSubtypeContributorRole filterItem) {

    PublicationSubtype publicationSubtype = this.itemService.findOne(id);
    List<PublicationSubtypeContributorRole> items = publicationSubtype.getPublicationSubtypeContributorRoles();
    List<PublicationSubtypeContributorRoleDTO> dtoItems = items.stream()
            .map(PublicationSubtypeContributorRoleDTO::new)
            .sorted(Comparator.comparingInt(PublicationSubtypeContributorRoleDTO::getSortValue))
            .collect(Collectors.toList());

    return new ResponseEntity<>(new Collection(dtoItems), HttpStatus.OK);
  }
}
