package ch.unige.aou.test.storage;

import java.nio.file.Path;
import java.nio.file.Paths;

import org.junit.jupiter.api.AfterEach;
import org.mockito.Mock;

import ch.unige.solidify.config.SolidifyProperties;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.util.FileTool;

import ch.unige.aou.AouMetadataVersion;
import ch.unige.aou.business.ContributorService;
import ch.unige.aou.business.LanguageService;
import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.PublicationService;
import ch.unige.aou.business.PublicationSubSubtypeService;
import ch.unige.aou.business.PublicationSubtypeService;
import ch.unige.aou.business.PublicationTypeService;
import ch.unige.aou.business.ResearchGroupService;
import ch.unige.aou.business.StructureService;
import ch.unige.aou.business.UserService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.service.MetadataService;
import ch.unige.aou.service.rest.DuplicateService;
import ch.unige.aou.service.rest.JournalTitleRemoteService;
import ch.unige.aou.service.storage.StorageService;

public abstract class StorageServiceTest {

  protected MetadataService metadataService;

  @Mock
  protected MessageService messageService;
  @Mock
  protected PublicationService publicationService;
  @Mock
  protected UserService userService;
  @Mock
  private PublicationTypeService publicationTypeService;
  @Mock
  private PublicationSubtypeService publicationSubtypeService;
  @Mock
  private PublicationSubSubtypeService publicationSubSubtypeService;
  @Mock
  private StructureService structureService;
  @Mock
  private ResearchGroupService researchGroupService;
  @Mock
  private ContributorService contributorService;
  @Mock
  private PersonService personService;
  @Mock
  private LanguageService languageService;
  @Mock
  private JournalTitleRemoteService journalTitleRemoteService;
  @Mock
  private DuplicateService duplicateService;

  protected Path storageDir = Paths.get("src", "test", "resources", "storage");

  protected StorageService storageService;

  protected Path workingDir = Paths.get("src", "test", "resources", "testing");

  public final static String TEST_DEPOSIT_JSON_FORM_DATA_TEMPLATE = "{\n"
          + "    \"contributors\": {\n"
          + "        \"contributors\": [\n"
          + "            {\n"
          + "                \"contributor\": {\n"
          + "                    \"cnIndividu\": \"12345\",\n"
          + "                    \"email\": \"john.doe@example.com\",\n"
          + "                    \"firstname\": \"John\",\n"
          + "                    \"institution\": \"unige\",\n"
          + "                    \"lastname\": \"Doe\",\n"
          + "                    \"orcid\": \"1234-1234-1234-1234\",\n"
          + "                    \"role\": \"author\"\n"
          + "                }\n"
          + "            }\n"
          + "        ]\n"
          + "    },\n"
          + "    \"description\": {\n"
          + "        \"container\": {\n"
          + "            \"identifiers\": {\n"
          + "                \"issn\": \"0028-0836\"\n"
          + "            },\n"
          + "            \"issue\": 8,\n"
          + "            \"title\": {\n"
          + "                \"lang\": \"eng\",\n"
          + "                \"text\": \"Nature\"\n"
          + "            }\n"
          + "        }\n"
          + "    },\n"
          + "    \"type\": {\n"
          + "        \"subsubtype\": \"A1-LETTRE\",\n"
          + "        \"subtype\": \"A1\",\n"
          + "        \"title\": \"%s\",\n"
          + "        \"type\": \"ARTICLE\"\n"
          + "    }\n"
          + "}";

  public void setUp() {
    AouProperties aouProperties = new AouProperties(new SolidifyProperties());
    aouProperties.setHome(this.workingDir.toAbsolutePath().toString());
    aouProperties.getStorage().setUrl(this.storageDir.toAbsolutePath().toUri().toString());

    this.metadataService = new MetadataService(aouProperties, this.messageService, this.publicationTypeService, this.publicationSubtypeService,
            this.publicationSubSubtypeService, this.structureService, this.researchGroupService, this.contributorService, this.languageService,
            this.journalTitleRemoteService, this.duplicateService, this.personService);

    if (!FileTool.ensureFolderExists(this.storageDir)) {
      throw new SolidifyRuntimeException("Cannot create " + this.storageDir);
    }
  }

  @AfterEach
  public void purgeData() {
    FileTool.deleteFolder(this.workingDir);
    FileTool.deleteFolder(this.storageDir);
  }

  public void storePublicationTest(Publication publication) throws Exception {
    this.storageService.checkMetadata(publication);
    this.storageService.generateExportMetadata(publication);
    this.storageService.storePublication(publication);
  }

  protected Publication getPublication(Path source, String aipName, AouMetadataVersion version) {
    final Publication publication = new Publication();
    return publication;
  }

  protected void runTests(String filePrefix) {
    for (final AouMetadataVersion version : AouMetadataVersion.values()) {
      try {
        this.storePublicationTest(this.getPublication(Paths.get("src", "test", "resources", "aip").toAbsolutePath(), filePrefix, version));
      } catch (final Exception e) {

      }
    }
  }

}
