package ch.unige.aou.service.imports;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

@Service
public class ArxivImportClientService {

  protected AbstractRestClientTool restClientTool;

  private Environment env;

  private String module;

  private String resourcePath;


  public ArxivImportClientService(AbstractRestClientTool restClientTool, Environment env) {
    this.env = env;
    this.restClientTool = restClientTool;
    this.module = ModuleName.ADMIN;
    this.resourcePath = SolidifyConstants.URL_SEP + ResourceName.EXTERNAL_DATA;
  }

  protected String getBaseUrl(String module) {
    String propertyName = "aou.module." + module + ".url";
    String url = this.env.getProperty(propertyName);
    if (url == null) {
      throw new IllegalStateException("Property aou.module." + module + ".url has to be defined.");
    }
    // Manage if there is a list of URLs
    if (url.contains(",")) {
      url = url.substring(0, url.indexOf(','));
    }
    return url;
  }

  public String importFileFromArxiv(String arxivId, String publicationId, String format) {
    String url = this.getBaseUrl(this.module) + this.resourcePath + SolidifyConstants.URL_SEP + AouActionName.IMPORT_FILE_ARXIV + "?arxivId="
            + arxivId + "&publicationId=" + publicationId + "&format=" + format ;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    return restTemplate.getForObject(url, String.class);
  }
}
