package ch.unige.aou.repository;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.unige.aou.model.settings.Institution;
import ch.unige.aou.model.settings.Person;

@Repository
@SuppressWarnings("squid:S1214")
public interface InstitutionRepository extends SolidifyRepository<Institution> {
  // *************
  // ** Queries **
  // *************

  @Query("SELECT p FROM Institution i JOIN i.people p WHERE i.resId = :parentId")
  public Page<Person> findPeopleByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT p FROM Institution i JOIN i.people p WHERE i.resId = :parentId AND p.resId=:id")
  public Person findPeopleByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT i FROM Institution i WHERE :emailSuffix IN i.emailSuffixes")
  public Institution findByEmailSuffix(@Param("emailSuffix") String emailSuffix);

  Institution findByName(String name);

  // ***************
  // ** Relations **
  // ***************

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public default <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass, Pageable pageable) {
    // Institution<->Person
    if (parentClass.equals(Institution.class) && childClass.equals(Person.class)) {
      return (Page<V>) this.findPeopleByParentId(parentId, pageable);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName() + ")");
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public default <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass, String id) {

    // Institution<->Person
    if (parentClass.equals(Institution.class) && childClass.equals(Person.class)) {
      return (V) this.findPeopleByParentId(parentId, id);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentIdAndChildId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName()
                    + ")");
  }

}
