package ch.unige.aou.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.unige.aou.model.settings.Institution;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.model.settings.Structure;

@Repository
public interface PersonRepository extends SolidifyRepository<Person> {

  // *************
  // ** Queries **
  // *************

  @Query("SELECT i"
          + " FROM Person p"
          + " JOIN p.institutions i"
          + " WHERE p.resId = :personId")
  Page<Institution> findInstitutionsByPersonId(@Param("personId") String personId, Pageable pageable);

  @Query("SELECT i"
          + " FROM Person p"
          + " JOIN p.institutions i"
          + " WHERE p.resId = :personId AND i.resId=:institutionId")
  Institution findInstitutionsByPersonId(@Param("personId") String personId, @Param("institutionId") String institutionId);

  @Query("SELECT r FROM Person p JOIN p.researchGroups r WHERE p.resId = :parentId AND r.resId=:id")
  ResearchGroup findResearchGroupByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT r FROM Person p JOIN p.researchGroups r WHERE p.resId = :parentId")
  Page<ResearchGroup> findResearchGroupByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT s FROM Person p JOIN p.structures s WHERE p.resId = :parentId AND s.resId=:id")
  Structure findStructureByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT s FROM Person p JOIN p.structures s WHERE p.resId = :parentId")
  Page<Structure> findStructureByParentId(@Param("parentId") String parentId, Pageable pageable);

  Person findByFirstName(String firstName);

  List<Person> findByFirstNameStartingWith(String firstNameStart);

  @Query("SELECT DISTINCT p FROM Person p"
                 + " INNER JOIN User u ON u.person.resId = p.resId"
                 + " WHERE p.firstName LIKE %:search% OR p.lastName LIKE %:search% OR p.orcid LIKE %:search%")
  Page<Person> searchAllWithUser(String search, Pageable pageable);

  @Query("SELECT p FROM Person p"
                 + " INNER JOIN User u ON u.externalUid = :userExternalUid")
  Person getPersonFromUser(String userExternalUid);

  // ***************
  // ** Relations **
  // ***************

  @Override
  @SuppressWarnings("unchecked")
  public default <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass, Pageable pageable) {
    // Person<->Institution
    if (parentClass.equals(Person.class) && childClass.equals(Institution.class)) {
      return (Page<V>) this.findInstitutionsByPersonId(parentId, pageable);
    }
    // ResearchGroup<->User
    if (parentClass.equals(Person.class) && childClass.equals(ResearchGroup.class)) {
      return (Page<V>) this.findResearchGroupByParentId(parentId, pageable);
    }
    // Structure<->User
    if (parentClass.equals(Person.class) && childClass.equals(Structure.class)) {
      return (Page<V>) this.findStructureByParentId(parentId, pageable);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName() + ")");
  }

  @Override
  @SuppressWarnings("unchecked")
  public default <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass, String childId) {

    // Person<->Institution
    if (parentClass.equals(Person.class) && childClass.equals(Institution.class)) {
      return (V) this.findInstitutionsByPersonId(parentId, childId);
    }

    // ResearchGroup<->User
    if (parentClass.equals(Person.class) && childClass.equals(ResearchGroup.class)) {
      return (V) this.findResearchGroupByParentId(parentId, childId);
    }
    // Structure<->User
    if (parentClass.equals(Person.class) && childClass.equals(Structure.class)) {
      return (V) this.findStructureByParentId(parentId, childId);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentIdAndChildId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName()
                    + ")");
  }


}
