package ch.unige.aou.test.admin;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.fail;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.http.HttpStatus;
import org.springframework.web.client.HttpClientErrorException;

import ch.unige.aou.model.security.Role;
import ch.unige.aou.test.AouTestConstants;

class RoleAsAdminIT extends AbstractAdminIT {

  @Override
  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  private static int TOTAL_NUMBER_OF_ROLES = 1;

  @Test
  void creationTest() {
    final Role role = new Role();
    role.setName(AouTestConstants.getRandomNameWithTemporaryLabel("Role"));

    try {
      this.roleService.create(role);
      fail("A FORBIDDEN Http response should be received");
    } catch (final HttpClientErrorException e) {
      assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());
    }
  }

  @Test
  void deleteTest() {
    final List<Role> actualRoles = this.roleService.findAll();

    // Delete
    final String resId = actualRoles.get(0).getResId();
    try {
      this.roleService.delete(resId);
      fail("A FORBIDDEN Http response should be received");
    } catch (final HttpClientErrorException e) {
      assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());
    }
  }

  @Test
  void findAllTest() {
    // Find all
    final List<Role> actualRoles = this.roleService.findAll();

    // Test all roles
    assertEquals(TOTAL_NUMBER_OF_ROLES, actualRoles.size());
  }

  @Test
  void updateTest() {
    // Get existing role
    Role localRole = this.roleService.findAll().get(0);

    // Update the role name
    final String newRoleName = AouTestConstants.getRandomNameWithTemporaryLabel("Role");
    localRole.setName(newRoleName);
    String localRoleId = localRole.getResId();
    HttpClientErrorException e = assertThrows(HttpClientErrorException.class, () -> {
      this.roleService.update(localRoleId, localRole);
    });
    assertEquals(HttpStatus.FORBIDDEN, e.getStatusCode());
  }
}
