package ch.unige.aou.model.index;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import ch.unige.solidify.rest.ResourceNoSql;
import ch.unige.solidify.util.JSONTool;

import ch.unige.aou.AouConstants;

@JsonDeserialize(using = PublicationMetadataDeserializer.class)
public class PublicationMetadata extends ResourceNoSql<PublicationMetadata> {

  private String index;
  private Map<String, Object> metadata = new HashMap<>();
  private String type;

  public PublicationMetadata() {
  }

  public PublicationMetadata(IndexType type) {
    this.type = type.toString().toLowerCase();
  }

  public void addLinksForAccess(WebMvcLinkBuilder linkBuilder) {
    super.addLinks(linkBuilder);
  }

  public boolean check() {
    return (this.getResId() != null && this.index != null && this.type != null && !this.metadata.isEmpty());
  }

  @Override
  public boolean equals(final Object other) {
    if (!(other instanceof PublicationMetadata)) {
      return false;
    }
    final PublicationMetadata castOther = (PublicationMetadata) other;
    return Objects.equals(this.index, castOther.index) && Objects.equals(this.type, castOther.type)
            && Objects.equals(this.metadata, castOther.metadata);
  }

  public String getIndex() {
    return this.index;
  }

  public Map<String, Object> getMetadata() {
    return this.metadata;
  }

  public Object getMetadata(String... xpath) {
    return this.getMetadata(this.metadata, xpath);
  }

  @JsonIgnore
  public String getMetadataVersion() {
    return (String) this.getMetadata(AouConstants.METADATA_VERSION_FIELD);
  }

  @JsonIgnore
  public String getDataCiteMetadata() {
    return (String) this.getMetadata(AouConstants.DATACITE_XML_FIELD);
  }

  public String getType() {
    return this.type;
  }

  @Override
  public int hashCode() {
    return Objects.hash(this.index, this.type, this.metadata);
  }

  public void setIndex(String index) {
    this.index = index;
  }

  public void setMetadata(Map<String, Object> metadata) {
    this.metadata = metadata;
  }

  public void setMetadata(String metadata) {
    this.metadata = JSONTool.convertJson2Map(metadata);
  }

  @Override
  public String toString() {
    return "ArchiveMetadata [resId=" + this.getResId() + ", index=" + this.index + ", type=" + this.type + ", metadata=" + this.metadata + "]";
  }

  @Override
  public boolean update(ResourceNoSql<PublicationMetadata> item) {
    // TODO Auto-generated method stub
    return true;
  }

  @SuppressWarnings("unchecked")
  private Object getMetadata(Map<String, Object> map, String... xpath) {
    if (map != null) {
      if (xpath.length == 1) {
        return map.get(xpath[0]);
      }
      return this.getMetadata((Map<String, Object>) map.get(xpath[0]), Arrays.copyOfRange(xpath, 1, xpath.length));
    } else {
      return null;
    }
  }

  private List<Object> getMetadataPropertiesList(String propertyPath) {
    final Object propertiesList = this.getMetadata(propertyPath.split("\\."));
    if (propertiesList != null && propertiesList instanceof ArrayList) {
      return (List<Object>) propertiesList;
    } else if (propertiesList != null) {
      // Only contains one element, that is the case of access_level restricted without embargo and
      // licence
      return Arrays.asList(propertiesList);
    } else {
      return new ArrayList<>();
    }
  }

}
