package ch.unige.aou.model.publication;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.settings.Label;

@Entity
public class PublicationSubSubtype extends AouResourceNormalized<PublicationSubSubtype> {

  @ManyToOne(targetEntity = PublicationSubtype.class)
  @NotNull
  private PublicationSubtype publicationSubtype;

  /**
   * SubSubtype name (e.g: Article, Article de données, Article Vidéo...)
   */
  @Size(min = 1)
  @NotNull
  @Column(nullable = false)
  private String name;

  /**
   * Property allowing to sort types in lists
   */
  @Column(nullable = false)
  private Integer sortValue;

  @ElementCollection
  @Column(name = AouConstants.DB_PUBLICATION_SUB_SUBTYPE_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_PUBLICATION_SUB_SUBTYPE_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_PUBLICATION_SUB_SUBTYPE_ID,
                           AouConstants.DB_LANGUAGE_ID }))
  private List<Label> descriptions = new ArrayList<>();

  @ElementCollection
  @Column(name = AouConstants.DB_PUBLICATION_SUB_SUBTYPE_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_PUBLICATION_SUB_SUBTYPE_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_PUBLICATION_SUB_SUBTYPE_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  /****************************************************************/

  public String getName() {
    return this.name;
  }

  public void setName(String name) {
    this.setCurrentProperty(name);
  }

  public Integer getSortValue() {
    return this.sortValue;
  }

  public void setSortValue(Integer sort) {
    this.setCurrentProperty(sort);
  }

  public PublicationSubtype getPublicationSubtype() {
    return this.publicationSubtype;
  }

  public void setPublicationSubtype(PublicationSubtype publicationSubtype) {
    this.setCurrentProperty(publicationSubtype);
  }

  public List<Label> getDescriptions() {
    return this.descriptions;
  }

  public void setDescriptions(List<Label> descriptions) {
    this.setCurrentProperty(descriptions);
  }
  public List<Label> getLabels() {
    return this.labels;
  }

  public void setLabels(Collection<Label> labels) {
    this.setCurrentProperty(labels);
  }

}
