package ch.unige.aou.model.publication;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;
import ch.unige.aou.model.settings.Label;
import ch.unige.aou.model.settings.ValidationRight;
import ch.unige.aou.rest.ResourceName;

@Entity
public class PublicationSubtype extends AouResourceNormalized<PublicationSubtype> {

  @ManyToOne(targetEntity = PublicationType.class)
  @NotNull
  private PublicationType publicationType;

  /**
   * Subtype name (e.g: Article scientifique, Chapitre de livre)
   */
  @Size(min = 1)
  @NotNull
  @Column(unique = true)
  private String name;

  /**
   * Subtype code. Used like a unique id.
   */
  @Size(min = 1)
  @NotNull
  @Column(unique = true, length = AouConstants.DB_SHORT_STRING_LENGTH)
  private String code;

  /**
   * Property allowing to sort subtypes in lists
   */
  @Column(nullable = false)
  private Integer sortValue;

  @JsonIgnore
  @OneToMany(mappedBy = PublicationSubtypeDocumentFileType.PATH_TO_PUBLICATION_SUBTYPE, cascade = CascadeType.ALL)
  private List<PublicationSubtypeDocumentFileType> publicationSubtypeDocumentFileTypes;

  @JsonIgnore
  @OneToMany(mappedBy = PublicationSubtypeContributorRole.PATH_TO_PUBLICATION_SUBTYPE, cascade = CascadeType.ALL)
  private List<PublicationSubtypeContributorRole> publicationSubtypeContributorRoles;

  @JsonIgnore
  @OneToMany(mappedBy = ValidationRight.PATH_TO_PUBLICATION_SUBTYPE, cascade = CascadeType.ALL, orphanRemoval = true)
  protected final List<ValidationRight> validationRights = new ArrayList<>();

  @ElementCollection
  @Column(name = AouConstants.DB_PUBLICATION_SUBTYPE_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_PUBLICATION_SUBTYPE_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_PUBLICATION_SUBTYPE_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> descriptions = new ArrayList<>();

  @ElementCollection
  @Column(name = AouConstants.DB_PUBLICATION_SUBTYPE_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_PUBLICATION_SUBTYPE_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_PUBLICATION_SUBTYPE_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  /****************************************************************/

  public PublicationType getPublicationType() {
    return this.publicationType;
  }

  public void setPublicationType(PublicationType publicationType) {
    this.setCurrentProperty(publicationType);
  }

  public String getName() {
    return this.name;
  }

  public void setName(String name) {
    this.setCurrentProperty(name);
  }

  public String getCode() {
    return this.code;
  }

  public void setCode(String code) {
    this.setCurrentProperty(code);
  }

  public Integer getSortValue() {
    return this.sortValue;
  }

  public void setSortValue(Integer sort) {
    this.setCurrentProperty(sort);
  }

  public List<PublicationSubtypeDocumentFileType> getPublicationSubtypeDocumentFileTypes() {
    return this.publicationSubtypeDocumentFileTypes;
  }

  public void setPublicationSubtypeDocumentFileTypes(
          List<PublicationSubtypeDocumentFileType> publicationSubtypeDocumentFileTypes) {
    this.setCurrentProperty(publicationSubtypeDocumentFileTypes);
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  public void setLabels(Collection<Label> labels) {
    this.setCurrentProperty(labels);
  }

  public boolean addPublicationSubtypeDocumentFileType(PublicationSubtypeDocumentFileType publicationSubtypeDocumentFileType) {
    publicationSubtypeDocumentFileType.setPublicationSubtype(this);
    return this.publicationSubtypeDocumentFileTypes.add(publicationSubtypeDocumentFileType);
  }

  public List<PublicationSubtypeContributorRole> getPublicationSubtypeContributorRoles() {
    return this.publicationSubtypeContributorRoles;
  }

  public void setPublicationSubtypeContributorRoles(
          List<PublicationSubtypeContributorRole> publicationSubtypeContributorRoles) {
    this.setCurrentProperty(publicationSubtypeContributorRoles);
  }

  public boolean addPublicationSubtypeContributorRole(PublicationSubtypeContributorRole publicationSubtypeContributorRole) {
    publicationSubtypeContributorRole.setPublicationSubtype(this);
    return this.publicationSubtypeContributorRoles.add(publicationSubtypeContributorRole);
  }

  public List<Label> getDescriptions() {
    return this.descriptions;
  }

  public void setDescriptions(List<Label> descriptions) {
    this.setCurrentProperty(descriptions);
  }

  @Override
  public void addLinks(WebMvcLinkBuilder linkBuilder, boolean mainRes, boolean subResOnly) {
    super.addLinks(linkBuilder, mainRes, subResOnly);
    if (mainRes) {
      this.add(linkBuilder.slash(this.getResId()).slash(ResourceName.PUBLICATION_SUB_SUBTYPES).withRel(ResourceName.PUBLICATION_SUB_SUBTYPES));
    }
  }
}
