package ch.unige.aou.specification;

import java.util.List;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.SolidifyConstants;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.settings.Person;

public class PublicationContributorSpecification extends PublicationSpecification {

  private String personId;
  private String cnIndividu;

  public PublicationContributorSpecification(Publication criteria, String personId, String cnIndividu) {
    super(criteria);
    this.cnIndividu = cnIndividu;
    this.personId = personId;
  }

  @Override
  protected void completePredicatesList(Root<Publication> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {

    super.completePredicatesList(root, query, builder, predicatesList);

    Path<Person> creatorPath = root.join("creator").get(SolidifyConstants.DB_RES_ID);
    Path<Person> contributorPath = root.join("contributors", JoinType.LEFT).get(AouConstants.DB_CN_INDIVIDU);

    Predicate equalPerson = builder.equal(creatorPath, this.personId);
    Predicate equalContributor = builder.equal(contributorPath, this.cnIndividu);

    //Get Publications that has the same creator OR the same cnIndividu, this because we want to have publications that we created OR that we contributed
    predicatesList.add(builder.or(equalPerson, equalContributor));

    query.distinct(true);
  }
}
