package ch.unige.aou.specification;

import java.time.OffsetDateTime;
import java.util.List;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.specification.SolidifySpecification;

import ch.unige.aou.model.settings.ResearchGroup;

public class ResearchGroupSpecification extends SolidifySpecification<ResearchGroup> {
  private static final long serialVersionUID = 8669419110874167742L;

  private OffsetDateTime lastUpdateBefore;

  public ResearchGroupSpecification(ResearchGroup criteria) {
    super(criteria);
  }

  @Override
  protected void completePredicatesList(Root<ResearchGroup> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {
    if (this.criteria.getName() != null) {
      predicatesList.add(builder.like(root.get("name"), "%" + this.criteria.getName() + "%"));
    }
    if (this.criteria.getCode() != null) {
      predicatesList.add(builder.like(root.get("code"), "%" + this.criteria.getCode() + "%"));
    }
    if (this.criteria.getAcronym() != null) {
      predicatesList.add(builder.like(root.get("acronym"), "%" + this.criteria.getAcronym() + "%"));
    }
    if (this.criteria.isActive() != null) {
      predicatesList.add(builder.equal(root.get("active"), this.criteria.isActive()));
    }
    if (this.criteria.isValidated() != null) {
      predicatesList.add(builder.equal(root.get("validated"), this.criteria.isValidated()));
    }
    if (this.lastUpdateBefore != null) {
      predicatesList.add(builder.greaterThanOrEqualTo(root.get("lastUpdate").get("when"), this.lastUpdateBefore));
    }
  }

  public OffsetDateTime getLastUpdateBefore() {
    return this.lastUpdateBefore;
  }

  public void setLastUpdateBefore(OffsetDateTime lastUpdateBefore) {
    this.lastUpdateBefore = lastUpdateBefore;
  }

}
