package ch.unige.aou.specification;

import java.util.List;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.specification.Join3TiersSpecification;

import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.model.settings.ValidationRight;

public class ValidationRightSpecification extends Join3TiersSpecification<ValidationRight> {

  public ValidationRightSpecification(ValidationRight joinCriteria) {
    super(joinCriteria, ValidationRight.PATH_TO_PERSON, ValidationRight.PATH_TO_STRUCTURE, ValidationRight.PATH_TO_PUBLICATION_SUBTYPE);
  }

  @Override
  protected String getParentId() {
    return this.joinCriteria.getPerson().getResId();
  }

  @Override
  protected String getChildId() {
    return this.joinCriteria.getStructure().getResId();
  }

  @Override
  protected String getGrandChildId() {
    return this.joinCriteria.getPublicationSubtype().getResId();
  }

  @Override
  protected void completeJoinPredicatesList(Root<ValidationRight> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {
    // do nothing, validation rights have no property in join table
  }

  /**
   * Filter on structure
   *
   * @param root
   * @param query
   * @param builder
   * @param predicatesList
   */
  @Override
  protected void completeChildPredicatesList(Root<ValidationRight> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {

    Structure structure = this.joinCriteria.getStructure();
    Path structurePath = this.getChildPath(root);

    if (structure.getParentStructure() != null && structure.getParentStructure().getResId() != null) {
      predicatesList.add(builder
              .equal(structurePath.get("parentStructure").get(SolidifyConstants.DB_RES_ID), structure.getParentStructure().getResId()));
    }
    if (structure.getName() != null) {
      predicatesList.add(builder.like(structurePath.get("name"), "%" + structure.getName() + "%"));
    }
    if (structure.getCnStructC() != null) {
      predicatesList.add(builder.equal(structurePath.get("cnStructC"), structure.getCnStructC()));
    }
    if (structure.getCodeStruct() != null) {
      predicatesList.add(builder.equal(structurePath.get("codeStruct"), structure.getCodeStruct()));
    }
    if (structure.getDewey() != null) {
      predicatesList.add(builder.equal(structurePath.get("dewey"), structure.getDewey()));
    }
    if (structure.getStatus() != null) {
      predicatesList.add(builder.equal(structurePath.get("status"), structure.getStatus()));
    }
    if (structure.getSortValue() != null) {
      predicatesList.add(builder.equal(structurePath.get("sortValue"), structure.getSortValue()));
    }
  }

}
