package ch.unige.aou.repository;

import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.Structure;

public interface StructureRepository extends SolidifyRepository<Structure> {
  Structure findByCnStructC(String cnStructC);

  Structure findByCodeStruct(String cStruct);

  List<Structure> findByNameAndParentStructureResId(String name, String parentStructureId);

  @Query("SELECT s.parentStructure.resId FROM Structure s WHERE s.resId = :resId")
  String getParentStructureResId(String resId);

  @Query("SELECT u FROM Structure s JOIN s.people u WHERE s.resId = :parentId AND u.resId=:id")
  Person findPersonByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT u FROM Structure s JOIN s.people u WHERE s.resId = :parentId")
  Page<Person> findPersonByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT s FROM User u JOIN u.person p JOIN p.structures s WHERE u.externalUid like :externalUid")
  Page<Structure> findByUserExternalUid(@Param("externalUid") String externalUid, Pageable pageable);

  @Query(value = "SELECT max(st.sortValue) FROM Structure st WHERE st.parentStructure.resId = :parentStructureId")
  Optional<Integer> findMaxSortValueByParentStructure(String parentStructureId);

  @Query(value = "SELECT max(st.sortValue) FROM Structure st WHERE st.parentStructure.resId IS NULL")
  Optional<Integer> getMaxSortValue();

  // ***************
  // ** Relations **
  // ***************

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public default <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass, Pageable pageable) {
    // ResearchGroup<->User
    if (parentClass.equals(Structure.class) && childClass.equals(Person.class)) {
      return (Page<V>) this.findPersonByParentId(parentId, pageable);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName() + ")");
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public default <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass, String id) {

    // ResearchGroup<->User
    if (parentClass.equals(Structure.class) && childClass.equals(Person.class)) {
      return (V) this.findPersonByParentId(parentId, id);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentIdAndChildId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName()
                    + ")");
  }
}
