package ch.unige.aou.service;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyProcessingException;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.util.FileTool;
import ch.unige.solidify.util.StringTool;
import ch.unige.solidify.util.XMLTool;

import ch.unige.aou.AouConstants;
import ch.unige.aou.AouMetadataVersion;
import ch.unige.aou.AouXmlNamespacePrefixMapper;
import ch.unige.aou.AouXmlNamespacePrefixMapperV1;
import ch.unige.aou.AouXmlNamespacePrefixMapperV2_0;
import ch.unige.aou.AouXmlNamespacePrefixMapperV2_1;
import ch.unige.aou.AouXmlNamespacePrefixMapperV2_2;
import ch.unige.aou.AouXmlNamespacePrefixMapperV2_3;
import ch.unige.aou.AouXmlNamespacePrefixMapperV2_4;
import ch.unige.aou.business.ContributorService;
import ch.unige.aou.business.LanguageService;
import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.PublicationSubSubtypeService;
import ch.unige.aou.business.PublicationSubtypeService;
import ch.unige.aou.business.PublicationTypeService;
import ch.unige.aou.business.ResearchGroupService;
import ch.unige.aou.business.StructureService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.exception.AoUMetadataParseException;
import ch.unige.aou.model.index.PublicationMetadata;
import ch.unige.aou.model.publication.DocumentFileType;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.service.metadata.MetadataExtractor;
import ch.unige.aou.service.metadata.MetadataExtractorV1;
import ch.unige.aou.service.metadata.MetadataExtractorV2;
import ch.unige.aou.service.metadata.MetadataValidator;
import ch.unige.aou.service.metadata.MetadataValidatorV1;
import ch.unige.aou.service.metadata.MetadataValidatorV2;
import ch.unige.aou.service.rest.DuplicateService;
import ch.unige.aou.service.rest.JournalTitleRemoteService;

@Service
public class MetadataService extends AouService {
  private static final Logger log = LoggerFactory.getLogger(MetadataService.class);

  private AouProperties aouProperties;
  private PublicationTypeService publicationTypeService;
  private PublicationSubtypeService publicationSubtypeService;
  private PublicationSubSubtypeService publicationSubSubtypeService;
  private StructureService structureService;
  private ResearchGroupService researchGroupService;
  private ContributorService contributorService;
  private LanguageService languageService;
  private JournalTitleRemoteService journalTitleRemoteService;
  private DuplicateService duplicateService;
  private PersonService personService;

  public MetadataService(AouProperties aouProperties, MessageService messageService, PublicationTypeService publicationTypeService,
          PublicationSubtypeService publicationSubtypeService,
          PublicationSubSubtypeService publicationSubSubtypeService, StructureService structureService,
          ResearchGroupService researchGroupService, ContributorService contributorService, LanguageService languageService,
          JournalTitleRemoteService journalTitleRemoteService, DuplicateService duplicateService, PersonService personService) {
    super(messageService);
    this.aouProperties = aouProperties;
    this.publicationTypeService = publicationTypeService;
    this.publicationSubtypeService = publicationSubtypeService;
    this.publicationSubSubtypeService = publicationSubSubtypeService;
    this.structureService = structureService;
    this.researchGroupService = researchGroupService;
    this.contributorService = contributorService;
    this.languageService = languageService;
    this.journalTitleRemoteService = journalTitleRemoteService;
    this.duplicateService = duplicateService;
    this.personService = personService;
  }

  public Object getOaiDcMetadata(PublicationMetadata archive) {
    // TODO Auto-generated method stub
    return null;
  }

  public PublicationMetadata getMetatadata(String resIdFromOaiId) {
    // TODO Auto-generated method stub
    return null;
  }

  public MetadataExtractor getMetadataExtractor(AouMetadataVersion version) {
    switch (version) {
      case V1_0:
        return new MetadataExtractorV1(this.aouProperties, this.publicationTypeService, this.publicationSubtypeService,
                this.publicationSubSubtypeService, this.structureService, this.researchGroupService, this.contributorService,
                this.languageService, this.journalTitleRemoteService, this.personService, this.messageService);
      case V2_0:
      case V2_1:
      case V2_2:
      case V2_3:
      case V2_4:
        return new MetadataExtractorV2(this.aouProperties, this.publicationTypeService, this.publicationSubtypeService,
                this.publicationSubSubtypeService, this.structureService, this.researchGroupService, this.contributorService,
                this.languageService, this.journalTitleRemoteService, this.personService, this.messageService);
      default:
        throw new SolidifyProcessingException(this.messageService.get("metadata.error.version", new Object[] { version }));
    }
  }

  public MetadataValidator getMetadataValidator(AouMetadataVersion version) {
    switch (version) {
      case V1_0:
        return new MetadataValidatorV1(this.aouProperties, (MetadataExtractorV1) this.getMetadataExtractor(version), this.messageService,
                this.publicationTypeService, this.publicationSubtypeService, this.publicationSubSubtypeService, this.researchGroupService,
                this.structureService, this.duplicateService);
      case V2_0:
      case V2_1:
      case V2_2:
      case V2_3:
      case V2_4:
        return new MetadataValidatorV2(this.aouProperties, (MetadataExtractorV2) this.getMetadataExtractor(version), this.messageService,
                this.publicationTypeService, this.publicationSubtypeService, this.publicationSubSubtypeService, this.researchGroupService,
                this.structureService, this.duplicateService);
      default:
        throw new SolidifyProcessingException(this.messageService.get("metadata.error.version", new Object[] { version }));
    }
  }

  public AouXmlNamespacePrefixMapper getAouXmlNamespacePrefixMapper(AouMetadataVersion version) {
    switch (version) {
      case V1_0:
        return new AouXmlNamespacePrefixMapperV1();
      case V2_0:
        return new AouXmlNamespacePrefixMapperV2_0();
      case V2_1:
        return new AouXmlNamespacePrefixMapperV2_1();
      case V2_2:
        return new AouXmlNamespacePrefixMapperV2_2();
      case V2_3:
        return new AouXmlNamespacePrefixMapperV2_3();
      case V2_4:
        return new AouXmlNamespacePrefixMapperV2_4();
      default:
        throw new SolidifyProcessingException(this.messageService.get("metadata.error.version", new Object[] { version }));
    }
  }

  public AouMetadataVersion detectVersionFromXmlMetadata(String xmlMetadata) {
    try {
      Document doc = XMLTool.parseXML(xmlMetadata);
      String namespace = doc.getNamespaceURI();

      for (AouMetadataVersion version : AouMetadataVersion.getAllVersions()) {
        AouXmlNamespacePrefixMapper mapper = this.getAouXmlNamespacePrefixMapper(version);
        if (XMLTool.getFirstNode(doc, "/" + AouConstants.XML_NAMESPACE_DEPOSIT_PREFIX + ":deposit_doc", mapper.getNamespacePrefixes()) != null) {
          return version;
        }
      }

      throw new SolidifyProcessingException("namespace '" + namespace + "' does not correspond to any metadata version");
    } catch (SAXException e) {
      throw new AoUMetadataParseException("Unable to parse metadata", e);
    } catch (ParserConfigurationException | IOException e) {
      throw new SolidifyRuntimeException("unable to detect metadata version", e);
    }
  }

  public String upgradeMetadataFormat(String metadata) {
    AouMetadataVersion startVersion = this.detectVersionFromXmlMetadata(metadata);
    AouMetadataVersion targetVersion = AouMetadataVersion.getDefaultVersion();

    if (startVersion == AouMetadataVersion.V1_0 && targetVersion == AouMetadataVersion.V2_0) {
      return this.transformFromV1ToV2_0(metadata);
    } else if (startVersion == AouMetadataVersion.V1_0 && targetVersion == AouMetadataVersion.V2_1) {
      return this.transformFromV1ToV2_1(metadata);
    } else if (startVersion == AouMetadataVersion.V1_0 && targetVersion == AouMetadataVersion.V2_2) {
      return this.transformFromV1ToV2_2(metadata);
    } else if (startVersion == AouMetadataVersion.V1_0 && targetVersion == AouMetadataVersion.V2_3) {
      return this.transformFromV1ToV2_3(metadata);
    } else if (startVersion == AouMetadataVersion.V1_0 && targetVersion == AouMetadataVersion.V2_4) {
      return this.transformFromV1ToV2_4(metadata);
    } else if (startVersion == AouMetadataVersion.V2_0 && targetVersion == AouMetadataVersion.V2_1) {
      return this.transformFromV2_0ToV2_1(metadata);
    } else if (startVersion == AouMetadataVersion.V2_0 && targetVersion == AouMetadataVersion.V2_2) {
      return this.transformFromV2_0ToV2_2(metadata);
    } else if (startVersion == AouMetadataVersion.V2_1 && targetVersion == AouMetadataVersion.V2_2) {
      return this.transformFromV2_1ToV2_2(metadata);
    } else if (startVersion == AouMetadataVersion.V2_0 && targetVersion == AouMetadataVersion.V2_3) {
      return this.transformFromV2_0ToV2_3(metadata);
    } else if (startVersion == AouMetadataVersion.V2_1 && targetVersion == AouMetadataVersion.V2_3) {
      return this.transformFromV2_1ToV2_3(metadata);
    } else if (startVersion == AouMetadataVersion.V2_2 && targetVersion == AouMetadataVersion.V2_3) {
      return this.transformFromV2_2ToV2_3(metadata);
    } else if (startVersion == AouMetadataVersion.V2_2 && targetVersion == AouMetadataVersion.V2_4) {
      return this.transformFromV2_2ToV2_4(metadata);
    } else if (startVersion == AouMetadataVersion.V2_3 && targetVersion == AouMetadataVersion.V2_4) {
      return this.transformFromV2_3ToV2_4(metadata);
    } else {
      throw new SolidifyRuntimeException("Upgrading metadata version from " + startVersion + " to " + targetVersion + " is not supported");
    }
  }

  public String transformFromV1ToV2_0(String metadataV1) {

    try {
      final ClassPathResource metadataV1ToV2XSLResource = new ClassPathResource(
              AouConstants.XSL_HOME + SolidifyConstants.URL_SEP + AouConstants.METADATA_V1_TO_V2_XSL);

      String xsl = FileTool.toString(metadataV1ToV2XSLResource.getInputStream());
      return XMLTool.transform(metadataV1, xsl);

    } catch (IOException e) {
      throw new SolidifyRuntimeException("unable to transform metadata from V1 to V2", e);
    }
  }

  /**
   * Difference is in the way structures are managed in Json: in 2.1, their values in an array of resId in Json
   * and the resId property must be filled in XML metadata
   *
   * @param metadataV2_0
   * @return
   */
  public String transformFromV2_0ToV2_1(String metadataV2_0) {
    AouXmlNamespacePrefixMapper aouXmlNamespacePrefixMapper = this.getAouXmlNamespacePrefixMapper(AouMetadataVersion.V2_0);
    XPath xpath = XMLTool.buildXPath(aouXmlNamespacePrefixMapper);
    try {
      Document document = XMLTool.parseXML(metadataV2_0);
      NodeList structuresList = (NodeList) xpath
              .compile("/aou_deposit:deposit_doc/aou_deposit:academic_structures/aou_deposit:academic_structure")
              .evaluate(document, XPathConstants.NODESET);
      for (int i = 0; i < structuresList.getLength(); i++) {
        Node structureNode = structuresList.item(i);
        Node codeNode = (Node) xpath.compile("./aou_deposit:code").evaluate(structureNode, XPathConstants.NODE);
        String codeStruct = codeNode.getTextContent();
        Structure structure = this.structureService.findByCodeStruct(codeStruct);
        Node resIdNode = document.createElementNS(AouConstants.METADATA_XML_NAMESPACE_V2, "resId");
        resIdNode.setTextContent(structure.getResId());
        structureNode.appendChild(resIdNode);
      }

      // update xmlns for V2.1
      String metadata = XMLTool.documentToString(document);
      metadata = metadata.replace("\"" + AouConstants.METADATA_XML_NAMESPACE_V2 + "\"", "\"" + AouConstants.METADATA_XML_NAMESPACE_V2_1 + "\"");
      return metadata;

    } catch (XPathExpressionException | ParserConfigurationException | SAXException | IOException | TransformerException e) {
      throw new SolidifyRuntimeException("Unable to transform metadata from V2.0 to V2.1", e);
    }
  }

  public String transformFromV2_1ToV2_2(String metadataV2_1) {
    AouXmlNamespacePrefixMapper aouXmlNamespacePrefixMapper = this.getAouXmlNamespacePrefixMapper(AouMetadataVersion.V2_1);
    XPath xpath = XMLTool.buildXPath(aouXmlNamespacePrefixMapper);
    try {
      Document document = XMLTool.parseXML(metadataV2_1);

      // transform pages
      Node pagesNode = (Node) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:pages").evaluate(document, XPathConstants.NODE);
      if (pagesNode != null) {
        Node pagesStartNode = (Node) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:pages/aou_deposit:start")
                .evaluate(document, XPathConstants.NODE);
        Node pagesEndNode = (Node) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:pages/aou_deposit:end")
                .evaluate(document, XPathConstants.NODE);
        String pagesStartValue = this.getTextContentIfNotNull(pagesStartNode);
        String pagesEndValue = this.getTextContentIfNotNull(pagesEndNode);
        String paging = "";

        if (!StringTool.isNullOrEmpty(pagesStartValue)) {
          paging = pagesStartValue;
        }

        if (!StringTool.isNullOrEmpty(pagesEndValue)) {
          if (!StringTool.isNullOrEmpty(paging)) {
            paging += "-";
          }
          paging += pagesEndValue;
        }

        // create new 'paging' node
        if (!StringTool.isNullOrEmpty(paging)) {
          Node pagingNode = document.createElementNS(AouConstants.METADATA_XML_NAMESPACE_V2_1, "paging");
          pagingNode.setTextContent(paging);
          pagesNode.appendChild(pagingNode);
        }

        // remove obsolete nodes 'start' and 'end'
        if (pagesStartNode != null) {
          pagesNode.removeChild(pagesStartNode);
        }
        if (pagesEndNode != null) {
          pagesNode.removeChild(pagesEndNode);
        }
      }

      // update xmlns for V2.2
      String metadata = XMLTool.documentToString(document);
      metadata = metadata.replace("\"" + AouConstants.METADATA_XML_NAMESPACE_V2_1 + "\"",
              "\"" + AouConstants.METADATA_XML_NAMESPACE_V2_2 + "\"");
      return metadata;

    } catch (XPathExpressionException | ParserConfigurationException | SAXException | IOException | TransformerException e) {
      throw new SolidifyRuntimeException("Unable to transform metadata from V2.1 to V2.2", e);
    }
  }

  public String transformFromV1ToV2_1(String metadataV1) {
    String metadataV2_0 = this.transformFromV1ToV2_0(metadataV1);
    return this.transformFromV2_0ToV2_1(metadataV2_0);
  }

  public String transformFromV1ToV2_2(String metadataV1) {
    String metadataV2_0 = this.transformFromV1ToV2_0(metadataV1);
    String metadataV2_1 = this.transformFromV2_0ToV2_1(metadataV2_0);
    return this.transformFromV2_1ToV2_2(metadataV2_1);
  }

  public String transformFromV1ToV2_3(String metadataV1) {
    String metadataV2_0 = this.transformFromV1ToV2_0(metadataV1);
    String metadataV2_1 = this.transformFromV2_0ToV2_1(metadataV2_0);
    String metadataV2_2 = this.transformFromV2_1ToV2_2(metadataV2_1);
    return this.transformFromV2_2ToV2_3(metadataV2_2);
  }

  public String transformFromV1ToV2_4(String metadataV1) {
    String metadataV2_3 = this.transformFromV1ToV2_3(metadataV1);
    return this.transformFromV2_3ToV2_4(metadataV2_3);
  }

  public String transformFromV2_0ToV2_2(String metadataV2_0) {
    String metadataV2_1 = this.transformFromV2_0ToV2_1(metadataV2_0);
    return this.transformFromV2_1ToV2_2(metadataV2_1);
  }

  public String transformFromV2_0ToV2_3(String metadataV2_0) {
    String metadataV2_1 = this.transformFromV2_0ToV2_1(metadataV2_0);
    String metadataV2_2 = this.transformFromV2_1ToV2_2(metadataV2_1);
    return this.transformFromV2_2ToV2_3(metadataV2_2);
  }

  public String transformFromV2_1ToV2_3(String metadataV2_1) {
    String metadataV2_2 = this.transformFromV2_1ToV2_2(metadataV2_1);
    return this.transformFromV2_2ToV2_3(metadataV2_2);
  }

  public String transformFromV2_2ToV2_3(String metadataV2_2) {
    AouXmlNamespacePrefixMapper aouXmlNamespacePrefixMapper = this.getAouXmlNamespacePrefixMapper(AouMetadataVersion.V2_1);
    XPath xpath = XMLTool.buildXPath(aouXmlNamespacePrefixMapper);
    try {
      Document document = XMLTool.parseXML(metadataV2_2);

      // transform container conference_title into conference_subtitle
      Node containerNode = (Node) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:container").evaluate(document, XPathConstants.NODE);
      Node conferenceTitleNode = (Node) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:container/aou_deposit:conference_title")
              .evaluate(document, XPathConstants.NODE);
      String conferenceTitle = this.getTextContentIfNotNull(conferenceTitleNode);

      if (containerNode != null && conferenceTitleNode != null && !StringTool.isNullOrEmpty(conferenceTitle)) {
        // create new Node
        Node conferenceSubtitleNode = document.createElementNS(AouConstants.METADATA_XML_NAMESPACE_V2_2, "conference_subtitle");
        conferenceSubtitleNode.setTextContent(conferenceTitle);
        containerNode.appendChild(conferenceSubtitleNode);

        // remove obsolete Node
        containerNode.removeChild(conferenceTitleNode);
      }

      // update xmlns for V2.3
      String metadata = XMLTool.documentToString(document);
      metadata = metadata.replace("\"" + AouConstants.METADATA_XML_NAMESPACE_V2_2 + "\"",
              "\"" + AouConstants.METADATA_XML_NAMESPACE_V2_3 + "\"");
      return metadata;

    } catch (XPathExpressionException | ParserConfigurationException | SAXException | IOException | TransformerException e) {
      throw new SolidifyRuntimeException("Unable to transform metadata from V2.2 to V2.3", e);
    }
  }

  public String transformFromV2_2ToV2_4(String metadataV2_2) {
    String metadataV2_3 = this.transformFromV2_2ToV2_3(metadataV2_2);
    return this.transformFromV2_3ToV2_4(metadataV2_3);
  }

  public String transformFromV2_3ToV2_4(String metadataV2_3) {
    AouXmlNamespacePrefixMapper aouXmlNamespacePrefixMapper = this.getAouXmlNamespacePrefixMapper(AouMetadataVersion.V2_3);
    XPath xpath = XMLTool.buildXPath(aouXmlNamespacePrefixMapper);
    try {
      Document document = XMLTool.parseXML(metadataV2_3);

      // transform document file types
      Map<String, String> fileTypeUpgradeMapping = new HashMap<>();
      fileTypeUpgradeMapping.put("Master", "Master thesis");
      fileTypeUpgradeMapping.put("Report (Published version)", DocumentFileType.REPORT);
      fileTypeUpgradeMapping.put("Report (Accepted version)", DocumentFileType.REPORT);
      fileTypeUpgradeMapping.put("Report (Submitted version)", DocumentFileType.REPORT);
      fileTypeUpgradeMapping.put("Working paper (Published version)", DocumentFileType.WORKING_PAPER);
      fileTypeUpgradeMapping.put("Working paper (Accepted version)", DocumentFileType.WORKING_PAPER);
      fileTypeUpgradeMapping.put("Working paper (Submitted version)", DocumentFileType.WORKING_PAPER);

      NodeList filesNode = (NodeList) xpath.compile("/aou_deposit:deposit_doc/aou_deposit:files/aou_deposit:file")
              .evaluate(document, XPathConstants.NODESET);
      for (int i = 0; i < filesNode.getLength(); i++) {
        Node fileNode = filesNode.item(i);

        Node fileTypeNode = (Node) xpath.compile("./aou_deposit:type").evaluate(fileNode, XPathConstants.NODE);
        String fileType = fileTypeNode.getTextContent();
        if (fileTypeUpgradeMapping.containsKey(fileType)) {
          fileTypeNode.setTextContent(fileTypeUpgradeMapping.get(fileType));
        }
      }

      // update xmlns for V2.4
      String metadata = XMLTool.documentToString(document);
      metadata = metadata.replace("\"" + AouConstants.METADATA_XML_NAMESPACE_V2_3 + "\"",
              "\"" + AouConstants.METADATA_XML_NAMESPACE_V2_4 + "\"");
      return metadata;

    } catch (XPathExpressionException | ParserConfigurationException | SAXException | IOException | TransformerException e) {
      throw new SolidifyRuntimeException("Unable to transform metadata from V2.3 to V2.4", e);
    }
  }

  private String getTextContentIfNotNull(Node node) {
    if (node != null && !StringTool.isNullOrEmpty(node.getTextContent())) {
      return node.getTextContent();
    }
    return null;
  }
}
