package ch.unige.aou.service.metadata;

import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;

import ch.unige.solidify.exception.SolidifyResourceNotFoundException;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.exception.SolidifyValidationException;
import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.util.StringTool;
import ch.unige.solidify.validation.ValidationError;

import ch.unige.aou.business.ContributorService;
import ch.unige.aou.business.LanguageService;
import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.PublicationSubSubtypeService;
import ch.unige.aou.business.PublicationSubtypeService;
import ch.unige.aou.business.PublicationTypeService;
import ch.unige.aou.business.ResearchGroupService;
import ch.unige.aou.business.StructureService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.AbstractContributor;
import ch.unige.aou.model.publication.CollaborationDTO;
import ch.unige.aou.model.publication.ContributorDTO;
import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.MetadataCollection;
import ch.unige.aou.model.publication.MetadataCorrection;
import ch.unige.aou.model.publication.MetadataDates;
import ch.unige.aou.model.publication.MetadataFile;
import ch.unige.aou.model.publication.MetadataFunding;
import ch.unige.aou.model.publication.MetadataLink;
import ch.unige.aou.model.publication.MetadataTextLang;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.settings.Language;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.model.tool.CleanTool;
import ch.unige.aou.model.xml.deposit.v2_4.Abstracts;
import ch.unige.aou.model.xml.deposit.v2_4.AcademicStructure;
import ch.unige.aou.model.xml.deposit.v2_4.AcademicStructures;
import ch.unige.aou.model.xml.deposit.v2_4.AccessLevel;
import ch.unige.aou.model.xml.deposit.v2_4.AuthorRole;
import ch.unige.aou.model.xml.deposit.v2_4.Classification;
import ch.unige.aou.model.xml.deposit.v2_4.Classifications;
import ch.unige.aou.model.xml.deposit.v2_4.Collaboration;
import ch.unige.aou.model.xml.deposit.v2_4.Collection;
import ch.unige.aou.model.xml.deposit.v2_4.Collections;
import ch.unige.aou.model.xml.deposit.v2_4.Container;
import ch.unige.aou.model.xml.deposit.v2_4.Contributor;
import ch.unige.aou.model.xml.deposit.v2_4.Contributors;
import ch.unige.aou.model.xml.deposit.v2_4.Correction;
import ch.unige.aou.model.xml.deposit.v2_4.Corrections;
import ch.unige.aou.model.xml.deposit.v2_4.Datasets;
import ch.unige.aou.model.xml.deposit.v2_4.DateWithType;
import ch.unige.aou.model.xml.deposit.v2_4.Dates;
import ch.unige.aou.model.xml.deposit.v2_4.DepositDoc;
import ch.unige.aou.model.xml.deposit.v2_4.DepositIdentifiers;
import ch.unige.aou.model.xml.deposit.v2_4.Embargo;
import ch.unige.aou.model.xml.deposit.v2_4.File;
import ch.unige.aou.model.xml.deposit.v2_4.FileType;
import ch.unige.aou.model.xml.deposit.v2_4.Files;
import ch.unige.aou.model.xml.deposit.v2_4.Funding;
import ch.unige.aou.model.xml.deposit.v2_4.Fundings;
import ch.unige.aou.model.xml.deposit.v2_4.Group;
import ch.unige.aou.model.xml.deposit.v2_4.Groups;
import ch.unige.aou.model.xml.deposit.v2_4.Keywords;
import ch.unige.aou.model.xml.deposit.v2_4.Languages;
import ch.unige.aou.model.xml.deposit.v2_4.Link;
import ch.unige.aou.model.xml.deposit.v2_4.LinkTypes;
import ch.unige.aou.model.xml.deposit.v2_4.Links;
import ch.unige.aou.model.xml.deposit.v2_4.ObjectFactory;
import ch.unige.aou.model.xml.deposit.v2_4.Pages;
import ch.unige.aou.model.xml.deposit.v2_4.Publisher;
import ch.unige.aou.model.xml.deposit.v2_4.Text;
import ch.unige.aou.model.xml.deposit.v2_4.TextLang;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.AbstractsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.AcademicsStructuresDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.ClassificationsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.CollectionsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.ContainerDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.ContributorsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.CorrectionsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.DatasetsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.DateWithTypeDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.DatesDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.DepositIdentifiersDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.FilesDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.FundingsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.GroupsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.KeywordsDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.LanguagesDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.LinksDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.PagesDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.PublisherDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.TextDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.deserializer.TextLangDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.AbstractsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.AcademicStructuresSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.AuthorRoleSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.ClassificationsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.CollectionsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.ContributorsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.CorrectionsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.DatasetsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.DateWithTypeSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.DatesSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.DepositDocSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.DepositIdentifiersSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.FilesSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.FundingsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.GroupsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.KeywordsSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.LanguagesSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.LinksSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.PagesSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.PublisherSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.TextLangSerializer;
import ch.unige.aou.model.xml.deposit.v2_4.serializer.TextSerializer;
import ch.unige.aou.service.rest.JournalTitleRemoteService;

@Service
public class MetadataExtractorV2 extends MetadataExtractor {

  ObjectFactory factory = new ObjectFactory();

  public MetadataExtractorV2(AouProperties aouProperties, PublicationTypeService publicationTypeService,
          PublicationSubtypeService publicationSubtypeService,
          PublicationSubSubtypeService publicationSubSubtypeService, StructureService structureService,
          ResearchGroupService researchGroupService, ContributorService contributorService, LanguageService languageService,
          JournalTitleRemoteService journalTitleRemoteService, PersonService personService, MessageService messageService) {
    super(aouProperties, publicationTypeService, publicationSubtypeService, publicationSubSubtypeService, structureService, researchGroupService,
            contributorService, languageService, journalTitleRemoteService, personService, messageService);
  }

  @Override
  public String transformFormDataToMetadata(String jsonData) {
    DepositDoc depositDoc = this.createDepositDocFromJsonFormData(jsonData);
    return this.serializeDepositDocToXml(depositDoc);
  }

  @Override
  public DepositDoc createDepositDocFromJsonFormData(String jsonData) {

    try {
      ObjectMapper mapper = this.getJsonFormDeserializerMapper();

      JsonNode root = mapper.readTree(jsonData);
      JsonNode typeNode = root.at("/type");
      JsonNode contributorsNode = root.at("/contributors");
      JsonNode descriptionNode = root.at("/description");

      DepositDoc depositDoc = mapper.readValue(typeNode.toString(), DepositDoc.class);
      ObjectReader updater = mapper.readerForUpdating(depositDoc);
      depositDoc = updater.readValue(contributorsNode.toString());
      updater = mapper.readerForUpdating(depositDoc);
      depositDoc = updater.readValue(descriptionNode.toString());

      this.replaceFormValuesByMetadataValues(depositDoc);

      return depositDoc;

    } catch (JsonProcessingException e) {
      if (e.getCause() instanceof SolidifyValidationException) {
        throw (SolidifyValidationException) e.getCause();
      }
      throw new SolidifyRuntimeException("unable to parse json data", e);
    }
  }

  @Override
  public void replaceFormValuesByMetadataValues(Object depositDocObj) {
    super.replaceFormValuesByMetadataValues(depositDocObj);
    this.completeStructures(depositDocObj);
    this.completeResearchGroups(depositDocObj);
  }

  private void completeStructures(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<Structure> structuresList = this.getStructures(depositDocObj);
    if (structuresList.size() > 0) {
      AcademicStructures academicStructures = new AcademicStructures();
      for (Structure structure : structuresList) {
        AcademicStructure academicStructure = new AcademicStructure();
        Structure structureDB = this.structureService.findById(structure.getResId()).orElse(null);
        academicStructure.setResId(structure.getResId());
        if (structureDB != null) {
          academicStructure.setCode(structureDB.getCodeStruct());
          academicStructure.setName(structureDB.getName());
        }
        academicStructures.getAcademicStructure().add(academicStructure);
      }
      depositDoc.setAcademicStructures(academicStructures);
    }
  }

  private void completeResearchGroups(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<ResearchGroup> researchGroupList = this.getResearchGroups(depositDocObj);
    if (researchGroupList.size() > 0) {
      Groups groups = new Groups();
      for (ResearchGroup researchGroup : researchGroupList) {
        Group group = new Group();
        ResearchGroup researchGroupDb = this.researchGroupService.findById(researchGroup.getResId()).orElse(null);
        group.setResId(researchGroup.getResId());
        if (researchGroupDb != null) {
          group.setName(researchGroupDb.getName());
          group.setCode(researchGroupDb.getCode() != null ? new BigInteger(researchGroupDb.getCode()) : null);
        }
        groups.getGroup().add(group);
      }
      depositDoc.setGroups(groups);
    }
  }

  @Override
  public String transformMetadataToFormData(String xmlData) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(xmlData);

    this.replaceMetadataValuesByFormValues(depositDoc);

    try {
      ObjectMapper mapper = this.getJsonFormSerializerMapper();
      return mapper.writeValueAsString(depositDoc);
    } catch (JsonProcessingException e) {
      throw new SolidifyRuntimeException("unable to transform XML data to Json form data", e);
    }
  }

  /**
   * 1) Save the document files properties into the metadata content
   * 2) Save the eventual thumbnail into metadata
   */
  @Override
  public void completeMetadataWithFiles(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    Files files = new Files();

    // add DocumentFiles into metadata
    for (DocumentFile df : publication.getDocumentFiles()) {
      File xmlFile = this.convertDocumentFileToXmlFile(df);
      files.getFile().add(xmlFile);
    }

    // add Thumbnail into metadata as Vignette
    if (publication.getThumbnail() != null) {
      File thumbnailFile = new File();
      thumbnailFile.setType(FileType.VIGNETTE);
      thumbnailFile.setAccessLevel(AccessLevel.PUBLIC);
      if (publication.getThumbnail().getFileSize() != null) {
        thumbnailFile.setSize(BigInteger.valueOf(publication.getThumbnail().getFileSize()));
      }
      thumbnailFile.setMimetype(publication.getThumbnail().getMimeType());
      thumbnailFile.setName(publication.getThumbnail().getFileName());
      files.getFile().add(thumbnailFile);
    }

    depositDoc.setFiles(files);
    //add into the publication metadata the info from file
    String xmlMetadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(xmlMetadata);
  }

  @Override
  public void completeClassificationsMetadataWithDeweyCodes(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    if (depositDoc.getAcademicStructures() != null && !depositDoc.getAcademicStructures().getAcademicStructure().isEmpty()) {
      Set<String> deweyCodes = new HashSet<>();
      for (AcademicStructure academicStructure : depositDoc.getAcademicStructures().getAcademicStructure()) {
        Structure structure = this.structureService.findByCodeStruct(academicStructure.getCode());
        if (structure != null && !StringTool.isNullOrEmpty(structure.getDewey())) {
          deweyCodes.add(structure.getDewey());
        }
      }

      if (!deweyCodes.isEmpty()) {
        depositDoc.setClassifications(new Classifications());
        for (String deweyCode : deweyCodes) {
          Classification classification = new Classification();
          classification.setCode("Dewey");
          classification.setItem(deweyCode);
          depositDoc.getClassifications().getClassification().add(classification);
        }

        String xmlMetadata = this.serializeDepositDocToXml(depositDoc);
        publication.setMetadata(xmlMetadata);
      }
    }
  }

  @Override
  protected void formatDepositDocDatesToFrontendFormat(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getDates() != null && depositDoc.getDates().getDate() != null) {
      List<DateWithType> dates = depositDoc.getDates().getDate();
      for (DateWithType date : dates) {
        if (!StringTool.isNullOrEmpty(date.getContent())) {
          String dateStr = this.updateDateFromMetadataToFrontendFormat(date.getContent());
          date.setContent(dateStr);
        }
      }
    }
  }

  @Override
  protected void formatDepositDocDatesToMetadataFormat(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getDates() != null && depositDoc.getDates().getDate() != null) {
      List<DateWithType> dates = depositDoc.getDates().getDate();
      for (DateWithType date : dates) {
        if (!StringTool.isNullOrEmpty(date.getContent())) {
          String dateStr = this.updateDateFromFrontendToMetadataFormat(date.getContent());
          date.setContent(dateStr);
        }
      }
    }
  }

  @Override
  public String serializeDepositDocToXml(Object depositDocObject) {
    try {
      DepositDoc depositDoc = (DepositDoc) depositDocObject;
      JAXBContext jaxbContext = JAXBContext.newInstance(DepositDoc.class);
      Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
      jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
      StringWriter sw = new StringWriter();
      jaxbMarshaller.marshal(depositDoc, sw);
      return sw.toString();

    } catch (Exception e) {
      throw new SolidifyRuntimeException("unable to serialize DepositDoc to XML", e);
    }
  }

  @Override
  public Object createDepositDocObjectFromXml(String xmlData) {
    if (xmlData == null) {
      throw new SolidifyValidationException(new ValidationError("XmlData is null. Unable to deserialize XML to DepositDoc."));
    }
    return JAXB.unmarshal(new StringReader(xmlData), DepositDoc.class);
  }

  @Override
  public void replaceIdsByTextValues(Object depositDocObj) {
    super.replaceIdsByTextValues(depositDocObj);

    /**
     * Languages
     */
    List<String> languageIds = this.getLanguages(depositDocObj);
    if (languageIds != null) {
      List<String> languageCodes = new ArrayList<>();
      for (String languageId : languageIds) {
        if (!StringTool.isNullOrEmpty(languageId)) {
          Optional<Language> language = this.languageService.findById(languageId);
          if (language.isPresent()) {
            languageCodes.add(language.get().getCode());
          } else {
            languageCodes.add(languageId);
          }
        } else {
          languageCodes.add(""); // add empty string, sometimes the front can send empty language
        }
      }
      this.setLanguages(depositDocObj, languageCodes);
    }
  }

  @Override
  public void replaceTextValuesByIds(Object depositDocObj) {
    super.replaceTextValuesByIds(depositDocObj);

    /**
     * Languages
     */
    List<String> languageStrList = this.getLanguages(depositDocObj);
    if (languageStrList != null) {
      List<String> languageIds = new ArrayList<>();
      for (String languageStr : languageStrList) {
        Language language = this.languageService.findByCode(languageStr);
        if (language != null) {
          languageIds.add(language.getResId());
        } else {
          languageIds.add(languageStr);
        }
      }
      this.setLanguages(depositDocObj, languageIds);
    }
  }

  @Override
  public String getPmcIdFromPublication(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());
    return this.getPmcId(depositDoc);
  }

  @Override
  public void addResearchGroup(ResearchGroup researchGroup, Publication publication) {
    DepositDoc depositDoc = this.createDepositDocFromJsonFormData(publication.getFormData());
    Group group = new Group();
    group.setResId(researchGroup.getResId());
    group.setName(researchGroup.getName());
    group.setCode(researchGroup.getCode() != null ? new BigInteger(researchGroup.getCode()) : null);

    this.ensureGroupExists(depositDoc);
    List<Group> groups = depositDoc.getGroups().getGroup();
    if (groups.stream().noneMatch(g -> g.getResId().equals(researchGroup.getResId()))) {
      groups.add(group);
    }

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public void addStructure(Structure structure, Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    // create a new AcademicStructure object
    AcademicStructure academicStructure = new AcademicStructure();
    academicStructure.setResId(structure.getResId());
    academicStructure.setCode(structure.getCodeStruct());
    academicStructure.setName(structure.getName());

    this.ensureAcademicStructuresExists(depositDoc);

    List<AcademicStructure> academicStructures = depositDoc.getAcademicStructures().getAcademicStructure();
    if (academicStructures.stream().noneMatch(acst -> acst.getResId().equals(structure.getResId()))) {
      academicStructures.add(academicStructure);
    }

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public void addLanguage(Language language, Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    this.ensureLanguagesExists(depositDoc);

    List<String> languages = depositDoc.getLanguages().getLanguage();
    if (languages.stream().noneMatch(lang -> lang.equals(language.getCode()))) {
      depositDoc.getLanguages().getLanguage().add(language.getCode());
    }

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public boolean isCnIndividuInContributors(Publication publication, String cnIndividu) {
    if (StringTool.isNullOrEmpty(cnIndividu)) {
      return false;
    }

    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());
    List<AbstractContributor> contributors = this.getContributors(depositDoc);
    return contributors.stream().anyMatch(abstractContributor -> abstractContributor instanceof ContributorDTO
            && cnIndividu.equals(((ContributorDTO) abstractContributor).getCnIndividu()));
  }

  @Override
  public void removeCnIndividuFromContributors(Publication publication, String cnIndividu) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    // loop on all contributors and replace cnIndividu by null if it matches
    List<AbstractContributor> contributors = this.getContributors(depositDoc);
    for (AbstractContributor abstractContributor : contributors) {
      if (abstractContributor instanceof ContributorDTO) {
        ContributorDTO contributorDTO = (ContributorDTO) abstractContributor;
        if (contributorDTO.getCnIndividu() != null && contributorDTO.getCnIndividu().equals(cnIndividu)) {
          contributorDTO.setCnIndividu(null);
        }
      }
    }
    this.setContributors(depositDoc, contributors);
    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  private void ensureGroupExists(DepositDoc depositDoc) {
    if (depositDoc.getGroups() == null) {
      depositDoc.setGroups(this.factory.createGroups());
    }
  }

  private void ensureAcademicStructuresExists(DepositDoc depositDoc) {
    if (depositDoc.getAcademicStructures() == null) {
      depositDoc.setAcademicStructures(this.factory.createAcademicStructures());
    }
  }

  private void ensureLanguagesExists(DepositDoc depositDoc) {
    if (depositDoc.getLanguages() == null) {
      depositDoc.setLanguages(this.factory.createLanguages());
    }
  }

  /*******************************************************************************************/

  @Override
  public void cleanMetadata(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    this.cleanTitle(depositDoc);
    this.cleanOriginalTitle(depositDoc);
    this.cleanMandator(depositDoc);
    this.cleanPublisher(depositDoc);
    this.cleanEdition(depositDoc);
    this.cleanAward(depositDoc);
    this.cleanAouCollection(depositDoc);
    this.cleanDoctorEmail(depositDoc);
    this.cleanDoctorAddress(depositDoc);
    this.cleanPublisherVersionUrl(depositDoc);
    this.cleanIdentifiers(depositDoc);
    this.cleanAbstracts(depositDoc);
    this.cleanNote(depositDoc);
    this.cleanKeywords(depositDoc);
    this.cleanCollections(depositDoc);
    this.cleanFundings(depositDoc);
    this.cleanDatasets(depositDoc);
    this.cleanLinks(depositDoc);
    this.cleanCorrections(depositDoc);
    this.cleanPages(depositDoc);
    this.cleanContainerValues(depositDoc);
    this.cleanContributors(depositDoc);

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  @Override
  public void cleanMetadataToClone(Publication publication) {
    DepositDoc depositDoc = (DepositDoc) this.createDepositDocObjectFromXml(publication.getMetadata());

    this.clearIdentifiers(depositDoc);
    this.clearFiles(depositDoc);

    String metadata = this.serializeDepositDocToXml(depositDoc);
    publication.setMetadata(metadata);
  }

  private void cleanTitle(DepositDoc depositDoc) {
    if (depositDoc.getTitle() != null && !StringTool.isNullOrEmpty(depositDoc.getTitle().getContent())) {
      String value = depositDoc.getTitle().getContent();
      value = CleanTool.cleanTitle(value);
      depositDoc.getTitle().setContent(value);
    }
  }

  private void cleanOriginalTitle(DepositDoc depositDoc) {
    if (depositDoc.getOriginalTitle() != null && !StringTool.isNullOrEmpty(depositDoc.getOriginalTitle().getContent())) {
      String value = depositDoc.getOriginalTitle().getContent();
      value = CleanTool.cleanTitle(value);
      depositDoc.getOriginalTitle().setContent(value);
    }
  }

  private void cleanMandator(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getMandator())) {
      String value = depositDoc.getMandator();
      value = CleanTool.cleanString(value);
      depositDoc.setMandator(value);
    }
  }

  private void cleanPublisher(DepositDoc depositDoc) {
    if (depositDoc.getPublisher() != null) {
      Publisher publisher = depositDoc.getPublisher();
      for (JAXBElement<String> element : publisher.getNameOrPlace()) {
        element.setValue(CleanTool.cleanString(element.getValue()));
      }
    }
  }

  private void cleanContainerValues(DepositDoc depositDoc) {
    if (depositDoc.getContainer() != null) {
      this.cleanContainerTitle(depositDoc);
      this.cleanEditor(depositDoc);
      this.cleanVolume(depositDoc);
      this.cleanIssue(depositDoc);
      this.cleanSpecialIssue(depositDoc);
      this.cleanConferenceSubtitle(depositDoc);
      this.cleanConferenceDate(depositDoc);
      this.cleanConferencePlace(depositDoc);
    }
  }

  private void cleanContainerTitle(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getTitle())) {
      depositDoc.getContainer().setTitle(CleanTool.cleanString(depositDoc.getContainer().getTitle()));
    }
  }

  private void cleanEditor(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getEditor())) {
      depositDoc.getContainer().setEditor(CleanTool.cleanString(depositDoc.getContainer().getEditor()));
    }
  }

  private void cleanVolume(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getVolume())) {
      depositDoc.getContainer().setVolume(CleanTool.cleanString(depositDoc.getContainer().getVolume()));
    }
  }

  private void cleanIssue(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getIssue())) {
      depositDoc.getContainer().setIssue(CleanTool.cleanString(depositDoc.getContainer().getIssue()));
    }
  }

  private void cleanSpecialIssue(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getSpecialIssue())) {
      depositDoc.getContainer().setSpecialIssue(CleanTool.cleanString(depositDoc.getContainer().getSpecialIssue()));
    }
  }

  private void cleanConferenceSubtitle(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getConferenceSubtitle())) {
      depositDoc.getContainer().setConferenceSubtitle(CleanTool.cleanString(depositDoc.getContainer().getConferenceSubtitle()));
    }
  }

  private void cleanConferenceDate(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getConferenceDate())) {
      depositDoc.getContainer().setConferenceDate(CleanTool.cleanString(depositDoc.getContainer().getConferenceDate()));
    }
  }

  private void cleanConferencePlace(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getContainer().getConferencePlace())) {
      depositDoc.getContainer().setConferencePlace(CleanTool.cleanString(depositDoc.getContainer().getConferencePlace()));
    }
  }

  private void cleanEdition(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getEdition())) {
      String value = depositDoc.getEdition();
      value = CleanTool.cleanString(value);
      depositDoc.setEdition(value);
    }
  }

  private void cleanAward(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getAward())) {
      String value = depositDoc.getAward();
      value = CleanTool.cleanString(value);
      depositDoc.setAward(value);
    }
  }

  private void cleanAouCollection(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getAouCollection())) {
      String value = depositDoc.getAouCollection();
      value = CleanTool.cleanString(value);
      depositDoc.setAouCollection(value);
    }
  }

  private void cleanDoctorEmail(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getDoctorEmail())) {
      String value = depositDoc.getDoctorEmail();
      value = CleanTool.cleanString(value);
      depositDoc.setDoctorEmail(value);
    }
  }

  private void cleanDoctorAddress(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getDoctorAddress())) {
      String value = depositDoc.getDoctorAddress();
      value = CleanTool.cleanString(value);
      depositDoc.setDoctorAddress(value);
    }
  }

  private void cleanPublisherVersionUrl(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getPublisherVersionUrl())) {
      String value = depositDoc.getPublisherVersionUrl();
      value = CleanTool.cleanString(value);
      depositDoc.setPublisherVersionUrl(value);
    }
  }

  private void cleanIdentifiers(DepositDoc depositDoc) {
    if (depositDoc.getIdentifiers() != null) {
      depositDoc.getIdentifiers().setDoi(CleanTool.cleanString(depositDoc.getIdentifiers().getDoi()));
      depositDoc.getIdentifiers().setArxiv(CleanTool.cleanString(depositDoc.getIdentifiers().getArxiv()));
      depositDoc.getIdentifiers().setIssn(CleanTool.cleanString(depositDoc.getIdentifiers().getIssn()));
      depositDoc.getIdentifiers().setDblp(CleanTool.cleanString(depositDoc.getIdentifiers().getDblp()));
      depositDoc.getIdentifiers().setIsbn(CleanTool.cleanStringAndHyphens(depositDoc.getIdentifiers().getIsbn(), true));
      depositDoc.getIdentifiers().setLocalNumber(CleanTool.cleanString(depositDoc.getIdentifiers().getLocalNumber()));
      depositDoc.getIdentifiers().setMmsid(CleanTool.cleanString(depositDoc.getIdentifiers().getMmsid()));
      depositDoc.getIdentifiers().setPmcid(CleanTool.cleanString(depositDoc.getIdentifiers().getPmcid()));
      depositDoc.getIdentifiers().setRepec(CleanTool.cleanString(depositDoc.getIdentifiers().getRepec()));
      depositDoc.getIdentifiers().setUrn(CleanTool.cleanString(depositDoc.getIdentifiers().getUrn()));
    }
  }

  private void cleanAbstracts(DepositDoc depositDoc) {
    if (depositDoc.getAbstracts() != null) {
      for (Text depositAbstract : depositDoc.getAbstracts().getAbstract()) {
        depositAbstract.setContent(CleanTool.cleanAbstract(depositAbstract.getContent()));
      }
    }
  }

  private void cleanNote(DepositDoc depositDoc) {
    if (!StringTool.isNullOrEmpty(depositDoc.getNote())) {
      String value = depositDoc.getNote();
      value = CleanTool.cleanString(value);
      depositDoc.setNote(value);
    }
  }

  private void cleanKeywords(DepositDoc depositDoc) {
    if (depositDoc.getKeywords() != null) {
      int i = 0;
      for (String keyword : depositDoc.getKeywords().getKeyword()) {
        depositDoc.getKeywords().getKeyword().set(i++, CleanTool.cleanString(keyword));
      }
    }
  }

  private void cleanCollections(DepositDoc depositDoc) {
    if (depositDoc.getCollections() != null) {
      for (Collection collection : depositDoc.getCollections().getCollection()) {
        collection.setName(CleanTool.cleanString(collection.getName()));
      }
    }
  }

  private void cleanFundings(DepositDoc depositDoc) {
    if (depositDoc.getFundings() != null) {
      for (Funding funding : depositDoc.getFundings().getFunding()) {
        funding.setName(CleanTool.cleanString(funding.getName()));
        funding.setCode(CleanTool.cleanString(funding.getCode()));
        funding.setAcronym(CleanTool.cleanString(funding.getAcronym()));
        funding.setJurisdiction(CleanTool.cleanString(funding.getJurisdiction()));
        funding.setProgram(CleanTool.cleanString(funding.getProgram()));
      }
    }
  }

  private void cleanDatasets(DepositDoc depositDoc) {
    if (depositDoc.getDatasets() != null) {
      int i = 0;
      for (String url : depositDoc.getDatasets().getUrl()) {
        depositDoc.getDatasets().getUrl().set(i++, CleanTool.cleanString(url));
      }
    }
  }

  private void cleanLinks(DepositDoc depositDoc) {
    if (depositDoc.getLinks() != null) {
      for (Link link : depositDoc.getLinks().getLink()) {
        link.getDescription().setContent(CleanTool.cleanString(link.getDescription().getContent()));
        link.setTarget(CleanTool.cleanString(link.getTarget()));
      }
    }
  }

  private void cleanCorrections(DepositDoc depositDoc) {
    if (depositDoc.getCorrections() != null) {
      for (Correction correction : depositDoc.getCorrections().getCorrection()) {
        correction.setNote(CleanTool.cleanString(correction.getNote()));
        correction.setDoi(CleanTool.cleanString(correction.getDoi()));
        correction.setPmid(CleanTool.cleanString(correction.getPmid()));
      }
    }
  }

  private void cleanPages(DepositDoc depositDoc) {
    if (depositDoc.getPages() != null) {
      for (JAXBElement<String> element : depositDoc.getPages().getPagingOrOther()) {
        element.setValue(CleanTool.cleanString(element.getValue()));
        if (element.getName().getLocalPart().equals("paging")) {
          element.setValue(CleanTool.cleanPaging(element.getValue()));
        }
      }
    }
  }

  private void cleanContributors(DepositDoc depositDoc) {
    if (depositDoc.getContributors() != null) {
      for (Serializable serializable : depositDoc.getContributors().getContributorOrCollaboration()) {
        if (serializable instanceof Contributor) {
          Contributor contributor = (Contributor) serializable;
          contributor.setFirstname(CleanTool.cleanString(contributor.getFirstname()));
          contributor.setLastname(CleanTool.cleanString(contributor.getLastname()));
          contributor.setEmail(CleanTool.cleanString(contributor.getEmail()));
          contributor.setInstitution(CleanTool.cleanString(contributor.getInstitution()));
          contributor.setOrcid(CleanTool.cleanString(contributor.getOrcid()));
          contributor.setCnIndividu(CleanTool.cleanString(contributor.getCnIndividu()));
        } else {
          Collaboration collaboration = (Collaboration) serializable;
          collaboration.setName(CleanTool.cleanString(collaboration.getName()));
        }
      }
    }
  }

  private void clearIdentifiers(DepositDoc depositDoc) {
    if (depositDoc.getIdentifiers() != null) {
      depositDoc.getIdentifiers().setDoi(null);
      depositDoc.getIdentifiers().setArxiv(null);
      depositDoc.getIdentifiers().setPmid(null);
      depositDoc.getIdentifiers().setPmcid(null);
    }
  }

  private void clearFiles(DepositDoc depositDoc) {
    if (depositDoc.getFiles() != null && depositDoc.getFiles().getFile() != null) {
      depositDoc.getFiles().getFile().clear();
      depositDoc.setFiles(null);
    }
  }

  /*******************************************************************************************/

  @Override
  public String getType(Object depositDocObj) {
    return ((DepositDoc) depositDocObj).getType();
  }

  @Override
  public void setType(Object depositDocObj, String value) {
    ((DepositDoc) depositDocObj).setType(value);
  }

  @Override
  public String getSubtype(Object depositDocObj) {
    return ((DepositDoc) depositDocObj).getSubtype();
  }

  @Override
  void setSubtype(Object depositDocObj, String value) {
    ((DepositDoc) depositDocObj).setSubtype(value);
  }

  @Override
  public String getSubSubtype(Object depositDocObj) {
    return ((DepositDoc) depositDocObj).getSubsubtype();
  }

  @Override
  void setSubSubtype(Object depositDocObj, String value) {
    ((DepositDoc) depositDocObj).setSubsubtype(value);
  }

  @Override
  public String getTitleContent(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getTitle() != null) {
      return depositDoc.getTitle().getContent();
    } else {
      return null;
    }
  }

  @Override
  public List<String> getLanguages(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getLanguages() != null) {
      return depositDoc.getLanguages().getLanguage();
    } else {
      return new ArrayList<>();
    }
  }

  void setLanguages(Object depositDocObj, List<String> languagesList) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    Languages languages = new Languages();
    languages.getLanguage().addAll(languagesList);
    depositDoc.setLanguages(languages);
  }

  @Override
  public String getTitleLang(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getTitle() != null) {
      return depositDoc.getTitle().getLang();
    } else {
      return null;
    }
  }

  @Override
  void setTitleLang(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getTitle() == null) {
      depositDoc.setTitle(new Text());
    }
    depositDoc.getTitle().setLang(value);
  }

  @Override
  public void setISSN(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() == null) {
      depositDoc.setIdentifiers(new DepositIdentifiers());
    }
    depositDoc.getIdentifiers().setIssn(value);
  }

  @Override
  public String getISBN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getIsbn();
    } else {
      return null;
    }
  }

  @Override
  public String getArxivId(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getArxiv();
    } else {
      return null;
    }
  }

  @Override
  public String getPmcId(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getPmcid();
    } else {
      return null;
    }
  }

  @Override
  public String getURN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getUrn();
    } else {
      return null;
    }
  }

  @Override
  public String getISSN(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getIssn();
    } else {
      return null;
    }
  }

  @Override
  public String getDOI(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getDoi();
    } else {
      return null;
    }
  }

  @Override
  public BigInteger getPmid(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getIdentifiers() != null) {
      return depositDoc.getIdentifiers().getPmid();
    } else {
      return null;
    }
  }

  @Override
  public List<AbstractContributor> getContributors(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<AbstractContributor> contributors = new ArrayList<>();
    if (depositDoc.getContributors() != null) {
      List<Serializable> contributorOrCollaborations = depositDoc.getContributors().getContributorOrCollaboration();
      for (Serializable serializable : contributorOrCollaborations) {
        if (serializable instanceof Contributor) {
          ContributorDTO contributor = new ContributorDTO();
          contributor.setFirstName(((Contributor) serializable).getFirstname());
          contributor.setLastName(((Contributor) serializable).getLastname());
          contributor.setEmail(((Contributor) serializable).getEmail());
          contributor.setCnIndividu(((Contributor) serializable).getCnIndividu());
          contributor.setStructure(((Contributor) serializable).getStructure());
          contributor.setInstitution(((Contributor) serializable).getInstitution());
          contributor.setOrcid(((Contributor) serializable).getOrcid());
          if (((Contributor) serializable).getRole() != null) {
            contributor.setRole(((Contributor) serializable).getRole().value());
          }
          contributors.add(contributor);
        } else {
          CollaborationDTO collaboration = new CollaborationDTO();
          collaboration.setName(((Collaboration) serializable).getName());
          contributors.add(collaboration);
        }
      }
    }
    return contributors;
  }

  @Override
  public void setContributors(Object depositDocObj, List<AbstractContributor> contributors) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getContributors() == null) {
      depositDoc.setContributors(this.factory.createContributors());
    }
    depositDoc.getContributors().getContributorOrCollaboration().clear();
    for (AbstractContributor abstractContributor : contributors) {
      if (abstractContributor instanceof ContributorDTO) {
        ContributorDTO contributorDTO = (ContributorDTO) abstractContributor;
        Contributor contributor = this.factory.createContributor();
        contributor.setRole(AuthorRole.fromValue(contributorDTO.getRole()));
        contributor.setFirstname(contributorDTO.getFirstName());
        contributor.setLastname(contributorDTO.getLastName());
        contributor.setEmail(contributorDTO.getEmail());
        contributor.setCnIndividu(contributorDTO.getCnIndividu());
        contributor.setInstitution(contributorDTO.getInstitution());
        contributor.setOrcid(contributorDTO.getOrcid());
        depositDoc.getContributors().getContributorOrCollaboration().add(contributor);
      } else if (abstractContributor instanceof CollaborationDTO) {
        CollaborationDTO collaborationDTO = (CollaborationDTO) abstractContributor;
        Collaboration collaboration = this.factory.createCollaboration();
        collaboration.setName(collaborationDTO.getName());
        depositDoc.getContributors().getContributorOrCollaboration().add(collaboration);
      }
    }
  }

  @Override
  public List<ResearchGroup> getResearchGroups(Object depositDocObj) {
    List<ResearchGroup> researchGroups = new ArrayList<>();
    Groups groups = ((DepositDoc) depositDocObj).getGroups();
    if (groups != null) {
      List<Group> groupList = groups.getGroup();
      for (Group g : groupList) {
        ResearchGroup researchGroup = new ResearchGroup();
        researchGroup.setResId(g.getResId());
        researchGroup.setName(g.getName());
        researchGroup.setCode(g.getCode() != null ? g.getCode().toString() : null);
        researchGroups.add(researchGroup);
      }
    }
    return researchGroups;
  }

  @Override
  public List<Structure> getStructures(Object depositDocObj) {
    List<Structure> structures = new ArrayList<>();
    AcademicStructures academicStructures = ((DepositDoc) depositDocObj).getAcademicStructures();
    if (academicStructures != null) {
      List<AcademicStructure> academicStructuresList = academicStructures.getAcademicStructure();
      for (AcademicStructure academicStructure : academicStructuresList) {
        Structure structure = new Structure();
        structure.setResId(academicStructure.getResId());
        structure.setName(academicStructure.getName() != null ? academicStructure.getName() : null);
        structure.setCodeStruct(academicStructure.getCode() != null ? academicStructure.getCode() : null);
        structures.add(structure);
      }
    }
    return structures;
  }

  @Override
  public List<MetadataDates> getDates(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataDates> metadataDates = new ArrayList<>();
    if (depositDoc.getDates() != null) {
      List<DateWithType> datesWithType = depositDoc.getDates().getDate();
      for (DateWithType date : datesWithType) {
        MetadataDates metadataDate = new MetadataDates();
        metadataDate.setContent(date.getContent());
        metadataDate.setType(date.getType().value());
        metadataDates.add(metadataDate);
      }
    }
    return metadataDates;
  }

  @Override
  String getOriginalTitleContent(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getOriginalTitle() != null) {
      return depositDoc.getOriginalTitle().getContent();
    }
    return null;
  }

  @Override
  String getOriginalTitleLang(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getOriginalTitle() != null) {
      return depositDoc.getOriginalTitle().getLang();
    }
    return null;
  }

  @Override
  void setOriginalTitleLang(Object depositDocObj, String value) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getOriginalTitle() == null) {
      depositDoc.setOriginalTitle(this.factory.createTextLang());
    }
    depositDoc.getOriginalTitle().setLang(value);
  }

  @Override
  List<MetadataTextLang> getAbstracts(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataTextLang> abstracts = new ArrayList<>();
    if (depositDoc.getAbstracts() != null && depositDoc.getAbstracts().getAbstract() != null) {
      for (Text abstractText : depositDoc.getAbstracts().getAbstract()) {
        abstracts.add(new MetadataTextLang(abstractText.getContent(), abstractText.getLang()));
      }
    }
    return abstracts;
  }

  @Override
  void setAbstracts(Object depositDocObj, List<MetadataTextLang> abstracts) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getAbstracts() == null) {
      depositDoc.setAbstracts(this.factory.createAbstracts());
    }
    depositDoc.getAbstracts().getAbstract().clear();
    for (MetadataTextLang textLang : abstracts) {
      Text text = this.factory.createText();
      text.setContent(textLang.getContent());
      text.setLang(textLang.getLang());
      depositDoc.getAbstracts().getAbstract().add(text);
    }
  }

  @Override
  List<MetadataLink> getLinks(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataLink> metadataLinks = new ArrayList<>();
    if (depositDoc.getLinks() != null && depositDoc.getLinks().getLink() != null) {
      for (Link link : depositDoc.getLinks().getLink()) {
        MetadataLink metadataLink = new MetadataLink();
        metadataLink.setTarget(link.getTarget());
        String description = link.getDescription() != null ? link.getDescription().getContent() : null;
        String lang = link.getDescription() != null ? link.getDescription().getLang() : null;
        metadataLink.setDescription(new MetadataTextLang(description, lang));
        if (link.getType() == LinkTypes.WEB) {
          metadataLink.setType(MetadataLink.LinkType.WEB);
        } else if (link.getType() == LinkTypes.INTERNAL) {
          metadataLink.setType(MetadataLink.LinkType.INTERNAL);
        } else if (link.getType() == null) {
          metadataLink.setType(null);
        }
        metadataLinks.add(metadataLink);
      }
    }
    return metadataLinks;
  }

  @Override
  void setLinks(Object depositDocObj, List<MetadataLink> links) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    if (depositDoc.getLinks() == null) {
      depositDoc.setLinks(this.factory.createLinks());
    }
    depositDoc.getLinks().getLink().clear();
    for (MetadataLink metadataLink : links) {
      Link link = this.factory.createLink();
      link.setTarget(metadataLink.getTarget());

      if (metadataLink.getDescription() != null) {
        Text text = this.factory.createText();
        text.setContent(metadataLink.getDescription().getContent());
        text.setLang(metadataLink.getDescription().getLang());
        link.setDescription(text);
      }

      if (metadataLink.getType() == MetadataLink.LinkType.WEB) {
        link.setType(LinkTypes.WEB);
      } else if (metadataLink.getType() == MetadataLink.LinkType.INTERNAL) {
        link.setType(LinkTypes.INTERNAL);
      }
      depositDoc.getLinks().getLink().add(link);
    }
  }

  @Override
  List<MetadataCollection> getCollections(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataCollection> collections = new ArrayList<>();
    if (depositDoc.getCollections() != null && depositDoc.getCollections().getCollection() != null) {
      for (Collection collection : depositDoc.getCollections().getCollection()) {
        collections.add(new MetadataCollection(collection.getName(), collection.getNumber()));
      }
    }
    return collections;
  }

  @Override
  public List<MetadataFunding> getFundings(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataFunding> metadataFundings = new ArrayList<>();
    if (depositDoc.getFundings() != null && depositDoc.getFundings().getFunding() != null) {
      for (Funding funding : depositDoc.getFundings().getFunding()) {
        MetadataFunding metadataFunding = new MetadataFunding();
        metadataFunding.setFunder(funding.getFunder() != null ? funding.getFunder() : null);
        metadataFunding.setName(funding.getName() != null ? funding.getName() : null);
        metadataFunding.setCode(funding.getCode() != null ? funding.getCode() : null);
        metadataFunding.setAcronym(funding.getAcronym() != null ? funding.getAcronym() : null);
        metadataFundings.add(metadataFunding);
      }
    }
    return metadataFundings;
  }

  @Override
  List<String> getDatasets(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<String> datasets = new ArrayList<>();
    if (depositDoc.getDatasets() != null && depositDoc.getDatasets().getUrl() != null) {
      for (String url : depositDoc.getDatasets().getUrl()) {
        datasets.add(!StringTool.isNullOrEmpty(url) ? url : null);
      }
    }
    return datasets;
  }

  @Override
  List<MetadataFile> getFiles(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataFile> files = new ArrayList<>();
    if (depositDoc.getFiles() != null && depositDoc.getFiles().getFile() != null) {
      for (File file : depositDoc.getFiles().getFile()) {
        MetadataFile metadataFile = new MetadataFile();
        metadataFile.setFileTypeValue(file.getType().value());
        metadataFile.setName(file.getName());
        metadataFile.setSize(file.getSize());
        metadataFile.setMimeType(file.getMimetype());
        metadataFile.setAccessLevel(file.getAccessLevel().value());
        metadataFile.setLabel(file.getLabel());
        metadataFile.setLicense(file.getLicense());
        files.add(metadataFile);
      }
    }
    return files;
  }

  @Override
  public List<MetadataCorrection> getCorrections(Object depositDocObj) {
    DepositDoc depositDoc = (DepositDoc) depositDocObj;
    List<MetadataCorrection> metadataCorrections = new ArrayList<>();
    if (depositDoc.getCorrections() != null && depositDoc.getCorrections().getCorrection() != null) {
      List<Correction> corrections = depositDoc.getCorrections().getCorrection();
      for (Correction correction : corrections) {
        MetadataCorrection metadataCorrection = new MetadataCorrection();
        metadataCorrection.setDoi(correction.getDoi());
        metadataCorrection.setPmid(correction.getPmid());
        metadataCorrection.setNote(correction.getNote());
        metadataCorrections.add(metadataCorrection);
      }
    }
    return metadataCorrections;
  }

  /*******************************************************************************************/

  private ObjectMapper getJsonFormDeserializerMapper() {
    SimpleModule module = new SimpleModule();
    module.addDeserializer(Abstracts.class, new AbstractsDeserializer());
    module.addDeserializer(AcademicStructures.class, new AcademicsStructuresDeserializer());
    module.addDeserializer(Classifications.class, new ClassificationsDeserializer());
    module.addDeserializer(Collections.class, new CollectionsDeserializer());
    module.addDeserializer(Container.class, new ContainerDeserializer());
    module.addDeserializer(Contributors.class, new ContributorsDeserializer());
    module.addDeserializer(Corrections.class, new CorrectionsDeserializer());
    module.addDeserializer(Datasets.class, new DatasetsDeserializer());
    module.addDeserializer(Dates.class, new DatesDeserializer());
    module.addDeserializer(DateWithType.class, new DateWithTypeDeserializer());
    module.addDeserializer(DepositIdentifiers.class, new DepositIdentifiersDeserializer());
    module.addDeserializer(Files.class, new FilesDeserializer());
    module.addDeserializer(Fundings.class, new FundingsDeserializer());
    module.addDeserializer(Groups.class, new GroupsDeserializer());
    module.addDeserializer(Keywords.class, new KeywordsDeserializer());
    module.addDeserializer(Languages.class, new LanguagesDeserializer());
    module.addDeserializer(Links.class, new LinksDeserializer());
    module.addDeserializer(Pages.class, new PagesDeserializer());
    module.addDeserializer(Publisher.class, new PublisherDeserializer());
    module.addDeserializer(TextLang.class, new TextLangDeserializer());
    module.addDeserializer(Text.class, new TextDeserializer());

    ObjectMapper mapper = new ObjectMapper();
    mapper.registerModule(module);
    mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    return mapper;
  }

  private ObjectMapper getJsonFormSerializerMapper() {
    SimpleModule module = new SimpleModule();
    module.addSerializer(Abstracts.class, new AbstractsSerializer());
    module.addSerializer(AcademicStructures.class, new AcademicStructuresSerializer());
    module.addSerializer(AuthorRole.class, new AuthorRoleSerializer());
    module.addSerializer(Classifications.class, new ClassificationsSerializer());
    module.addSerializer(Collections.class, new CollectionsSerializer());
    module.addSerializer(Contributors.class, new ContributorsSerializer());
    module.addSerializer(Corrections.class, new CorrectionsSerializer());
    module.addSerializer(Datasets.class, new DatasetsSerializer());
    module.addSerializer(Dates.class, new DatesSerializer());
    module.addSerializer(DateWithType.class, new DateWithTypeSerializer());
    module.addSerializer(DepositDoc.class, new DepositDocSerializer());
    module.addSerializer(DepositIdentifiers.class, new DepositIdentifiersSerializer());
    module.addSerializer(Files.class, new FilesSerializer());
    module.addSerializer(Fundings.class, new FundingsSerializer());
    module.addSerializer(Groups.class, new GroupsSerializer());
    module.addSerializer(Keywords.class, new KeywordsSerializer());
    module.addSerializer(Languages.class, new LanguagesSerializer());
    module.addSerializer(Links.class, new LinksSerializer());
    module.addSerializer(Pages.class, new PagesSerializer());
    module.addSerializer(Publisher.class, new PublisherSerializer());
    module.addSerializer(Text.class, new TextSerializer());
    module.addSerializer(TextLang.class, new TextLangSerializer());

    ObjectMapper mapper = new ObjectMapper();
    mapper.registerModule(module);
    mapper.enable(SerializationFeature.INDENT_OUTPUT);
    return mapper;
  }

  private File convertDocumentFileToXmlFile(DocumentFile documentFile) {
    try {
      File file = new File();
      file.setType(FileType.fromValue(documentFile.getDocumentFileType().getValue()));
      file.setAccessLevel(AccessLevel.fromValue(
              documentFile.getAccessLevel().toString().substring(0, 1).toUpperCase() + documentFile.getAccessLevel().toString().substring(1)
                      .toLowerCase()));
      file.setName(documentFile.getFileName());
      if (documentFile.getFileSize() != null) {
        file.setSize(BigInteger.valueOf(documentFile.getFileSize()));
      }
      file.setMimetype(documentFile.getMimetype());
      if (!StringTool.isNullOrEmpty(documentFile.getLabel())) {
        file.setLabel(documentFile.getLabel());
      }
      if (documentFile.getLicense() != null) {
        file.setLicense(documentFile.getLicense().getOpenLicenseId());
      }
      if (documentFile.getEmbargoEndDate() != null) {
        Embargo embargo = new Embargo();
        embargo.setAccessLevel(AccessLevel.fromValue(
                documentFile.getEmbargoAccessLevel().toString().substring(0, 1).toUpperCase() + documentFile.getEmbargoAccessLevel().toString()
                        .substring(1).toLowerCase()));
        embargo.setEndDate(DatatypeFactory.newInstance().newXMLGregorianCalendar(documentFile.getEmbargoEndDate().toString()));
        file.setEmbargo(embargo);
      }
      return file;
    } catch (DatatypeConfigurationException e) {
      throw new SolidifyResourceNotFoundException(
              "'Unable to convert embargo date'" + documentFile.getEmbargoEndDate() + "'to XMLGregorianCalendar format'");
    }
  }
}
