package ch.unige.aou.service.rest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.CollectionPage;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.rest.UnigePersonDTO;

@Service
@Profile("person-search-aou-prod")
public class AoUProdUnigePersonSearchService extends UnigePersonSearchService {

  private static final int PAGE_SIZE = 10;

  private String apiUrl;

  private String apiKey;

  public AoUProdUnigePersonSearchService(AouProperties aouProperties) {
    this.apiUrl = aouProperties.getMetadata().getSearch().getPerson().getUrl();
    this.apiKey = aouProperties.getMetadata().getSearch().getPerson().getApiKey();
  }

  @Override
  protected Collection<UnigePersonDTO> getResultsFromExternalService(String firstname, String lastname) {
    if (!StringTool.isNullOrEmpty(this.apiUrl) && !StringTool.isNullOrEmpty(this.apiKey)) {

      try {
        RestTemplateBuilder restTemplateBuilder = new RestTemplateBuilder();
        RestTemplate client = restTemplateBuilder.rootUri(this.apiUrl).build();

        final String jsonString = client.postForObject(this.apiUrl, this.getRequest(firstname, lastname), String.class);

        final ObjectMapper mapper = new ObjectMapper();
        final JavaType listOfMaps = mapper.getTypeFactory().constructParametricType(List.class, Map.class);
        List<Map> results = mapper.readValue(jsonString, listOfMaps);

        List<UnigePersonDTO> resultList = new ArrayList<>();

        results.forEach(map -> {
          String cnIndividu = this.getStringOrNull(map, "cn");
          String mapFirstname = this.getStringOrNull(map, "firstname");
          String mapLastname = this.getStringOrNull(map, "lastname");
          String yearOfBirthStr = this.getStringOrNull(map, "birthday");
          Integer yearOfBirth = null;
          if (!StringTool.isNullOrEmpty(yearOfBirthStr)) {
            yearOfBirth = Integer.parseInt(yearOfBirthStr);
          }
          String function = this.getStringOrNull(map, "fonction");
          String structure = this.getStringOrNull(map, "structure");

          UnigePersonDTO unigePersonDTO = new UnigePersonDTO(cnIndividu, mapFirstname, mapLastname, yearOfBirth, function, structure);
          resultList.add(unigePersonDTO);
        });
        CollectionPage page = new CollectionPage(0, PAGE_SIZE, 1, resultList.size());
        return new Collection<>(resultList, page);
      } catch (final IOException e) {
        throw new SolidifyRuntimeException(e.getMessage());
      }
    } else {
      throw new SolidifyRuntimeException("incomplete configuration");
    }
  }

  private HttpEntity<MultiValueMap<String, String>> getRequest(String firstname, String lastname) {
    MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();
    parameters.add("firstname", firstname);
    parameters.add("lastname", lastname);

    HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
    headers.set("X-API-KEY", this.apiKey);

    return new HttpEntity<>(parameters, headers);
  }

  private String getStringOrNull(Map map, String propertyName) {
    String value = map.getOrDefault(propertyName, "").toString();
    if (StringTool.isNullOrEmpty(value)) {
      return null;
    } else {
      return value;
    }
  }
}
