package ch.unige.aou.test;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.StringReader;
import javax.xml.bind.JAXB;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import ch.unige.aou.AouMetadataVersion;
import ch.unige.aou.model.xml.deposit.v2_4.DepositDoc;
import ch.unige.aou.service.MetadataService;
import ch.unige.aou.service.metadata.MetadataExtractorV1;

class MetadataUpgradeTest extends MetadataTest {

  private MetadataService metadataService;

  @Override
  @BeforeEach
  public void setUp() {
    super.setUp();
    this.metadataService = new MetadataService(this.aouProperties, this.messageService, this.publicationTypeService,
            this.publicationSubtypeService, this.publicationSubSubtypeService, this.structureService, this.researchGroupService,
            this.contributorService, this.languageService, this.journalTitleRemoteService, this.duplicateService, this.personService);
  }

  @Test
  void upgradeMetadataFromV1toDefaultVersion() {

    try {
      /**
       * Create DepositDoc from Json form
       */
      String jsonFormData = this.getJsonFormData(AouMetadataVersion.V1_0);
      MetadataExtractorV1 metadataExtractorV1 = (MetadataExtractorV1) this.metadataService.getMetadataExtractor(AouMetadataVersion.V1_0);
      ch.unige.aou.model.xml.deposit.v1.DepositDoc depositDocV1 = metadataExtractorV1.createDepositDocFromJsonFormData(jsonFormData);
      assertNotNull(depositDocV1);
      String metadata_v1_xml = metadataExtractorV1.serializeDepositDocToXml(depositDocV1);

      String metadata_default_version_xml = this.metadataService.upgradeMetadataFormat(metadata_v1_xml);
      assertNotNull(metadata_default_version_xml);

      DepositDoc depositDocDefaultVersion = JAXB.unmarshal(new StringReader(metadata_default_version_xml), DepositDoc.class);
      this.assertEqualValuesDefaultVersion(depositDocDefaultVersion, true);

    } catch (Exception e) {
      fail(String.format("unable to upgrade metadata from %s to %s", AouMetadataVersion.V1_0, AouMetadataVersion.getDefaultVersion()), e);
    }
  }

  @Test
  void upgradeMetadataFromV2_3toDefaultVersion() {
    AouMetadataVersion startVersion = AouMetadataVersion.V2_3;
    try {
      /**
       * Create DepositDoc from Json form
       */
      String metadata_start_version_xml = this.getXmlMetadata(startVersion);
      String metadata_default_version_xml = this.metadataService.upgradeMetadataFormat(metadata_start_version_xml);
      assertNotNull(metadata_default_version_xml);

      DepositDoc depositDocDefaultVersion = JAXB.unmarshal(new StringReader(metadata_default_version_xml), DepositDoc.class);
      this.assertEqualValuesDefaultVersion(depositDocDefaultVersion, true);

    } catch (Exception e) {
      fail(String.format("unable to upgrade metadata from %s to %s", startVersion, AouMetadataVersion.getDefaultVersion()), e);
    }
  }
}
